@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2006 the xine-project
+ * Copyright (C) 2003-2007 the xine-project
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -16,12 +16,11 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
  * USA
  *
- * $Id: engine.c,v 1.36 2006/04/04 22:39:24 dsalt Exp $
- *
  * init xine engine, set up script engine
  */
 
 #include "globals.h"
+#include "version.h"
 
 #include <stdio.h>
 #include <stdlib.h>
@@ -29,6 +28,9 @@
 #include <pthread.h>
 #include <stdarg.h>
 #include <errno.h>
+#include <unistd.h>
+
+#include <glib/gthread.h>
 
 #include "engine.h"
 #include "ui.h"
@@ -36,7 +38,7 @@
 
 se_t *gse; /* global script engine */
 xine_t *xine; /* global xine instance */
-pthread_mutex_t engine_lock = PTHREAD_RECURSIVE_MUTEX_INITIALIZER_NP;
+GStaticRecMutex engine_lock = G_STATIC_REC_MUTEX_INIT;
 
 gboolean initialised = FALSE;
 
@@ -53,16 +55,12 @@ static gboolean modified = FALSE;
 GAsyncQueue *js_queue;
 static gboolean queue_available = FALSE;
 
-#if GXINE_GTK_OLDER_THAN(2,8,0)
-WEAK (void, gtk_about_dialog_set_wrap_license);
-#endif
-
 static JSBool
 show_about (JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
 {
   static const gchar *const authors[] = {
-    "Guenter Bartsch <guenter@users.sourceforge.net>",
     "Darren Salt <dsalt@users.sourceforge.net>",
+    "Guenter Bartsch <guenter@users.sourceforge.net>",
     NULL
   };
   static const gchar *const documenters[] = {
@@ -81,8 +79,8 @@ show_about (JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
     GtkBox *vbox;
     /* translators' names go here */
     const gchar *translator = _("TRANSLATOR");
-    gchar *licence = GTK_HAVE (gtk_about_dialog_set_wrap_license, 2, 8, 0)
-      ? _("This program is free software; you can redistribute it and/or "
+    gchar *licence = 
+	_("This program is free software; you can redistribute it and/or "
 	  "modify it under the terms of the GNU General Public Licence as "
 	  "published by the Free Software Foundation; either version 2 of the "
 	  "Licence, or (at your option) any later version.\n"
@@ -95,46 +93,36 @@ show_about (JSContext *cx, JSObject *obj, uintN argc, jsval *argv, jsval *rval)
 	  "You should have received a copy of the GNU General Public Licence "
 	  "along with this program; if not, write to the Free Software "
 	  "Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, "
-	  "MA  02110-1301  USA")
-      /* Translators: rewrap as required */
-      : _("This program is free software; you can\n"
-	  "redistribute it and/or modify it under the terms of\n"
-	  "the GNU General Public Licence as published by\n"
-	  "the Free Software Foundation; either version 2 of\n"
-	  "the Licence, or (at your option) any later version.\n"
-	  "\n"
-	  "This program is distributed in the hope that it will\n"
-	  "be useful, but WITHOUT ANY WARRANTY; without\n"
-	  "even the implied warranty of MERCHANTABILITY\n"
-	  "or FITNESS FOR A PARTICULAR PURPOSE. See the\n"
-	  "GNU General Public Licence for more details.\n"
-	  "\n"
-	  "You should have received a copy of the GNU\n"
-	  "General Public Licence along with this program;\n"
-	  "if not, write to the Free Software Foundation, Inc.,\n"
-	  "51 Franklin St, Fifth Floor, Boston,\n"
 	  "MA  02110-1301  USA");
 
     about = gtk_about_dialog_new ();
     g_object_set (G_OBJECT (about),
 		  "name", "gxine",
 		  "version", VERSION,
-		  "copyright", _("© 2002-2006 the xine project team"),
+		  "copyright", get_copyright_notice (),
 		  "comments", _("A GTK media player front end for xine-lib"),
 		  "license", licence,
+		  "wrap-license", TRUE,
 		  "website", "http://xinehq.de/",
 		  "authors", authors,
 		  "documenters", documenters,
 		  "translator-credits",
 		  strcmp (translator, "TRANSLATOR") ? translator : NULL,
 		  "logo-icon-name", GXINE_LOGO, NULL);
-    if (GTK_HAVE (gtk_about_dialog_set_wrap_license, 2, 8, 0))
-      gtk_about_dialog_set_wrap_license ((GtkAboutDialog *) about, TRUE);
 
-    label = gtk_label_new ("<small><i>" VENDOR_PKG_VERSION "</i></small>");
-    gtk_label_set_use_markup (GTK_LABEL (label), TRUE);
+    char *version_label =
+      g_strdup_printf (_("<small><i>%s\nUsing xine-lib %s, built with %s</i></small>"),
+		       VENDOR_PKG_VERSION CSET_ID,
+		       xine_get_version_string (), XINE_VERSION);
+    label = ui_label_new_with_markup (version_label);
+    gtk_label_set_justify (label, GTK_JUSTIFY_CENTER);
+    free (version_label);
     vbox = GTK_BOX (GTK_DIALOG (about)->vbox);
     gtk_box_pack_end (vbox, label, FALSE, FALSE, 2);
+    g_object_connect (about,
+	"signal::delete-event", G_CALLBACK (gtk_widget_hide_on_delete), NULL,
+	"signal::response", G_CALLBACK (gtk_widget_hide), NULL,
+	NULL);
   }
 
   window_show (about, NULL);
@@ -148,6 +136,7 @@ print_cb (void *user_data, const char *str, ...)
   if (verbosity)
   {
     va_list ap;
+    /* JS diagnostic output */
     g_print (_("engine: < "));
     va_start (ap, str);
     vprintf (str, ap);
@@ -172,10 +161,10 @@ int engine_exec_obj (const char *cmd, se_o_t *obj, se_print_cb_t cb,
     g_warning ("JS execution during startup: %s: %s", src, cmd);
 #endif
 
-  if (pthread_mutex_lock (&engine_lock))
+  if (!g_static_rec_mutex_trylock (&engine_lock))
   {
     g_critical (_("JS execution attempted while lock already held.\n"
-		  "Try to reproduce this in a debugger (set a breakpoint on g_critical).\n"
+		  "Try to reproduce this in a debugger (set a breakpoint on g_log).\n"
 		  "Get a backtrace of all gxine threads and report it!"));
     engine_queue_push (cmd, NULL, cb, cb_data, ecb, src);
     return 1;
@@ -194,6 +183,7 @@ int engine_exec_obj (const char *cmd, se_o_t *obj, se_print_cb_t cb,
 
   if ((str = se_result_str (gse)))
   {
+    /* JS output. Could end up in a terminal or the JS console. */
     str = g_strdup_printf (_("result: %s"), str);
     cb (cb_data, "%s", str);
     free (str);
@@ -224,11 +214,11 @@ int engine_exec_obj (const char *cmd, se_o_t *obj, se_print_cb_t cb,
   else
   {
     /* return 0 iff the result type wasn't recognised */
-    pthread_mutex_unlock (&engine_lock);
+    g_static_rec_mutex_unlock (&engine_lock);
     return 0;
   }
 
-  pthread_mutex_unlock (&engine_lock);
+  g_static_rec_mutex_unlock (&engine_lock);
   return 1;
 }
 
@@ -300,10 +290,10 @@ static gboolean js_queue_cb (GtkWidget *widget, GdkEventClient *event,
   /* js_lock must be recursive since, if the engine is busy, this function
    * WILL be re-entered from the same thread with the lock already held.
    */
-  static pthread_mutex_t js_lock = PTHREAD_RECURSIVE_MUTEX_INITIALIZER_NP;
+  static GStaticRecMutex js_lock = G_STATIC_REC_MUTEX_INIT;
   exec_t *js;
 
-  pthread_mutex_lock (&js_lock);
+  g_static_rec_mutex_lock (&js_lock);
   while ((js = g_async_queue_try_pop (js_queue)))
   {
     /* This function is entered with the GDK lock held. This is bad, from
@@ -322,7 +312,7 @@ static gboolean js_queue_cb (GtkWidget *widget, GdkEventClient *event,
     free (js->src);
     free (js);
   }
-  pthread_mutex_unlock (&js_lock);
+  g_static_rec_mutex_unlock (&js_lock);
   return TRUE;
 }
 
@@ -333,7 +323,7 @@ load_user_startup_script (void)
   char *text = read_entire_file (file, NULL);
   if (!text)
   {
-    text = strdup ("");
+    text = strdup ("\n");
     if (errno != ENOENT)
       display_error (FROM_GXINE, _("Couldn't load startup script"), "%s: %s",
 		     file, strerror (errno));
@@ -436,6 +426,7 @@ void engine_init (void)
   xine = xine_new ();
 
   cfgfilename = get_config_filename (FILE_CONFIG);
+  have_config = !access (cfgfilename, R_OK); /* upgrade or install? */
   xine_config_load (xine, cfgfilename);
   free (cfgfilename);
   xine_engine_set_param (xine, XINE_ENGINE_PARAM_VERBOSITY, verbosity);
@@ -498,6 +489,7 @@ engine_startup_cb (gpointer data)
   g_signal_connect (G_OBJECT (app), "client-event",
 		    G_CALLBACK (js_queue_cb), NULL);
   js_queue_cb (NULL, NULL, NULL);
+  gxine_init_done ();
   return FALSE;
 }
 
@@ -531,7 +523,7 @@ engine_startup_script (void)
   se_startup_cmds = load_user_startup_script ();
   se_eval (gse, se_startup_cmds, NULL, NULL, NULL, _("Startup script"));
 
-  gtk_init_add ((GtkFunction) engine_startup_cb, NULL);
+  gxine_init_add ((GtkFunction) engine_startup_cb, NULL);
 }
 
 void
@@ -556,37 +548,47 @@ save_startup_script (void)
 #ifdef LOCK_DEBUG
 #include <assert.h>
 
-static __thread int m_depth = 0;
+static __thread int m_depth = 0, g_depth = 0;
+static pthread_mutex_t guard = PTHREAD_MUTEX_INITIALIZER;
 
 #include <sys/syscall.h>
-_syscall0 (pid_t, gettid);
+#include <unistd.h>
+#define gettid() (int) syscall (__NR_gettid)
 
 int gxine_mutex_lock (pthread_mutex_t *mutex, const char *fn, int line)
 {
-  int l = m_depth++;
+  (pthread_mutex_lock) (&guard);
+  int l = m_depth++ + g_depth;
+  (pthread_mutex_unlock) (&guard);
   g_printerr ("%*s   lock %5d: %s %d\n", l + 3, "pth", gettid (), fn, line);
   return (pthread_mutex_lock) (mutex);
 }
 
 int gxine_mutex_unlock (pthread_mutex_t *mutex, const char *fn, int line)
 {
-  int l = --m_depth;
-  assert (l >= 0);
+  (pthread_mutex_lock) (&guard);
+  int l = --m_depth + g_depth;
+  assert (m_depth >= 0);
+  (pthread_mutex_unlock) (&guard);
   g_printerr ("%*s unlock %5d: %s %d\n", l + 3, "pth", gettid (), fn, line);
   return (pthread_mutex_unlock) (mutex);
 }
 
 void gxine_threads_enter (const char *fn, int line)
 {
-  int l = m_depth++;
+  (pthread_mutex_lock) (&guard);
+  int l = g_depth++ + m_depth;
+  (pthread_mutex_unlock) (&guard);
   g_printerr ("%*s   lock %5d: %s %d\n", l + 3, "gdk", gettid (), fn, line);
   (gdk_threads_enter) ();
 }
 
 void gxine_threads_leave (const char *fn, int line)
 {
-  int l = --m_depth;
-  assert (l >= 0);
+  (pthread_mutex_lock) (&guard);
+  int l = --g_depth + m_depth;
+  /* assert (l >= 0); ** -1 unavoidably occurs in one instance */
+  (pthread_mutex_unlock) (&guard);
   g_printerr ("%*s unlock %5d: %s %d\n", l + 3, "gdk", gettid (), fn, line);
   (gdk_threads_leave) ();
 }
