@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2005 the xine-project
+ * Copyright (C) 2002-2007 the xine-project
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -16,8 +16,6 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
  * USA
  *
- * $Id: info_widgets.c,v 1.19 2006/03/27 22:11:24 dsalt Exp $
- *
  * nice black information display areas
  */
 
@@ -35,7 +33,9 @@
 
 #include "i18n.h"
 #include "defs.h"
+#include "gtkvideo.h"
 #include "info_widgets.h"
+#include "ui.h"
 #include "utils.h"
 #include "playlist.h"
 #include "player.h"
@@ -54,7 +54,13 @@ typedef gboolean (*gxineinfo_cb) (gpointer);
 struct gxineinfo_private_s
 {
   gxineinfo_cb postinit, update;
-  GtkWidget *line[MAX_LINES];
+  GtkLabel *line[MAX_LINES];
+  int data;
+  struct {
+    int time, length, tcount, tno, ccount, cno;
+    gboolean remaining, seekable;
+  } prev;
+  gboolean force;
 };
 typedef struct gxineinfo_private_s gxineinfo_private_t;
 
@@ -121,8 +127,14 @@ gxineinfo_get_type (void)
   static GtkType type = 0;
 
   if (!type)
+  {
     type = g_type_register_static (GTK_TYPE_ALIGNMENT, "GtkGxineInfo",
 				   &info, (GTypeFlags) 0);
+    xine_config_register_bool
+      (xine, "gui.show_time_remaining", 0,
+       _("If set, show the length of the remaining part of the stream."),
+       NULL, 0, gtk_true, NULL);
+  }
   return type;
 }
 
@@ -139,7 +151,8 @@ pad_table (GtkWidget *table, int l, int r, int t, int b, int pad)
 
 static GtkWidget *
 gxineinfo_new (guint height, PangoEllipsizeMode ellipsize, guint width,
-	       gxineinfo_cb postinit, gxineinfo_cb update, const char *name)
+	       gxineinfo_cb postinit, gxineinfo_cb update, const char *name,
+	       guint buttons)
 {
   GtkWidget *self, *table;
   unsigned int i;
@@ -159,6 +172,7 @@ gxineinfo_new (guint height, PangoEllipsizeMode ellipsize, guint width,
   priv = GET_PRIVATE (self);
   priv->postinit = postinit;
   priv->update = update;
+  priv->data = 0;
 
   gtk_widget_push_composite_child ();
 
@@ -174,13 +188,16 @@ gxineinfo_new (guint height, PangoEllipsizeMode ellipsize, guint width,
 
   for (i = 0; i < height; ++i)
   {
-    GtkWidget *line = priv->line[i] = gtk_label_new (NULL);
+    GtkWidget *ev = (buttons & 1 << i) ? gtk_event_box_new () : NULL;
+    GtkWidget *line = ui_label_new_with_xalign ("", 0);
+    priv->line[i] = GTK_LABEL (line);
     gtk_widget_set_name (line, i ? small : large);
-    gtk_misc_set_alignment (GTK_MISC (line), 0, 0.5);
     gtk_label_set_ellipsize (GTK_LABEL (line), ellipsize);
     if (!i && width)
       gtk_label_set_width_chars (GTK_LABEL (line), width);
-    gtk_table_attach (GTK_TABLE (table), line, 1, 2, i + 1, i + 2,
+    if (ev)
+      gtk_container_add (GTK_CONTAINER (ev), line);
+    gtk_table_attach (GTK_TABLE (table), ev ? : line, 1, 2, i + 1, i + 2,
 		      GTK_FILL | GTK_EXPAND, GTK_FILL, 0, 0);
   }
   for (; i < MAX_LINES; ++i)
@@ -219,7 +236,7 @@ gxineinfo_update_line (GSList *objs, guint lineno, const char *format, ...)
   {
     gxineinfo_private_t *priv = GET_PRIVATE (objs->data);
     if (priv->line[lineno])
-      gtk_label_set_text (GTK_LABEL (priv->line[lineno]), text);
+      gtk_label_set_text (priv->line[lineno], text);
   }
   free (text);
 }
@@ -349,7 +366,7 @@ infobar_show_metadata (GSList * bars)
 GtkWidget *
 create_infobar (gboolean small)
 {
-  return gxineinfo_new (small ? 1 : 3, PANGO_ELLIPSIZE_END, 0, NULL, NULL, "info");
+  return gxineinfo_new (small ? 1 : 3, PANGO_ELLIPSIZE_END, 0, NULL, NULL, "info", 0);
 }
 
 /*
@@ -367,69 +384,152 @@ update_time_widget_cb (gpointer data)
     gchar full[132];
   } time;
 
-  pthread_mutex_lock (&widgets_update_lock);
+  if (pthread_mutex_trylock (&widgets_update_lock))
+    return TRUE;
   gdk_threads_enter ();
 
   time.length[0] = time.current[0] = 0;
 
-  if (xine_get_status (stream) == XINE_STATUS_PLAY)
+  xine_cfg_entry_t entry;
+  gboolean state = priv->data ^
+		   !!(xine_config_lookup_entry
+			(xine, "gui.show_time_remaining", &entry)
+		      && entry.num_value);
+  gboolean seekable = xine_get_stream_info (stream, XINE_STREAM_INFO_SEEKABLE);
+  gtk_widget_set_state ((GtkWidget *)widget, state ? GTK_STATE_ACTIVE : GTK_STATE_NORMAL);
+
+  if (xine_get_status (stream) == XINE_STATUS_PLAY && !playlist_showing_logo ())
   {
     if (!xine_get_pos_length (stream, &pos_stream, &pos_time, &length_time))
       pos_stream = pos_time = length_time = 0;
+
+    if (state)
+      pos_time = length_time - pos_time;
+
     pos_time -= pos_time % 1000;
+    if (!priv->force && pos_time == priv->prev.time
+	&& length_time == priv->prev.length && state == priv->prev.remaining
+	&& seekable == priv->prev.seekable)
+      goto finish; /* no need to redisplay the time */
+
+    priv->prev.time = pos_time;
+    priv->prev.length = length_time;
+    priv->prev.remaining = state;
+    priv->prev.seekable = seekable;
+
     int_to_timestring (pos_time, time.current, sizeof (time.current));
+
     length_time = round_second (length_time);
     if (length_time > 0)
       int_to_timestring (length_time, time.length, sizeof (time.length));
     else
       snprintf (time.length, sizeof (time.length), "%s",
-		xine_get_stream_info (stream, XINE_STREAM_INFO_SEEKABLE)
-		  ? _("??:??:??") : _("live"));
+		seekable ? _("??:??:??") : _("live"));
   }
+  else
+  {
+    if (!priv->force && priv->prev.time == -1 && priv->prev.length == -1)
+      goto finish; /* no need to redisplay the (lack of) time */
+    priv->prev.time = priv->prev.length = -1;
+    strcpy (time.current, "•");
+  }
+
   if (priv->line[1])
   {
-    gtk_label_set_text (GTK_LABEL (priv->line[0]), time.current);
-    gtk_label_set_text (GTK_LABEL (priv->line[1]), time.length);
+    gtk_label_set_text (priv->line[0], time.current);
+    gtk_label_set_text (priv->line[1], time.length);
   }
   else if (time.current[0])
   {
     strcat (time.full, " / ");
     memmove (time.full + strlen (time.full), time.length, sizeof (time.length));
-    gtk_label_set_text (GTK_LABEL (priv->line[0]), time.full);
+    gtk_label_set_text (priv->line[0], time.full);
   }
   else
-    gtk_label_set_text (GTK_LABEL (priv->line[0]), "");
+    gtk_label_set_text (priv->line[0], "");
 
+finish:
 #ifdef XINE_STREAM_INFO_DVD_TITLE_NUMBER
   time.full[0] = 0;
-  int count = xine_get_stream_info (stream, XINE_STREAM_INFO_DVD_TITLE_COUNT);
-  if (count) /* "T" = DVD title number */
-    sprintf (time.full, _("T%d/%d "),
-	     xine_get_stream_info (stream, XINE_STREAM_INFO_DVD_TITLE_NUMBER),
-	     count);
-  count = xine_get_stream_info (stream, XINE_STREAM_INFO_DVD_CHAPTER_COUNT);
-  if (count) /* "C" = DVD chapter number */
-    sprintf (time.full + strlen (time.full), _("C%d/%d "),
-	     xine_get_stream_info (stream, XINE_STREAM_INFO_DVD_CHAPTER_NUMBER),
-	     count);
-  gtk_label_set_text (GTK_LABEL (priv->line[2]), time.full);
+  int tcount = xine_get_stream_info (stream, XINE_STREAM_INFO_DVD_TITLE_COUNT);
+  int tno = tcount ? xine_get_stream_info (stream, XINE_STREAM_INFO_DVD_TITLE_NUMBER) : 0;
+  int ccount = xine_get_stream_info (stream, XINE_STREAM_INFO_DVD_CHAPTER_COUNT);
+  int cno = ccount ? xine_get_stream_info (stream, XINE_STREAM_INFO_DVD_CHAPTER_NUMBER) : 0;
+
+  if (priv->force || tcount != priv->prev.tcount || tno != priv->prev.tno
+		  || ccount != priv->prev.ccount || cno != priv->prev.cno)
+  {
+    priv->prev.tcount = tcount;
+    priv->prev.tno = tno;
+    priv->prev.ccount = ccount;
+    priv->prev.cno = cno;
+    if (tcount) /* "T" = DVD title number */
+      sprintf (time.full, _("T%d/%d "), tno, tcount);
+    if (ccount) /* "C" = DVD chapter number */
+      sprintf (time.full + strlen (time.full), _("C%d/%d "), cno, ccount);
+    gtk_label_set_text (priv->line[2], time.full);
+  }
 #endif
 
+  priv->force = FALSE;
   gdk_threads_leave ();
   pthread_mutex_unlock (&widgets_update_lock);
+  return FALSE;
+}
+
+static gboolean
+frob_format_cb (GtkWidget *widget, GdkEventButton *event, gpointer data)
+{
+  if (event->button == 1)
+  {
+    gxineinfo_private_t *priv = GET_PRIVATE (data);
+    int flag = (priv->data ^= 1);
+    priv->force = TRUE;
+
+    GList **listp = g_object_get_data ((GObject *) data, "gxine-chain");
+    if (listp)
+    {
+      GList *item = *listp;
+      while (item)
+      {
+	priv = GET_PRIVATE (item->data);
+	priv->data = flag;
+	priv->force = TRUE;
+	item = item->next;
+      }
+      gxineinfo_update (*listp);
+    }
+    else
+      update_time_widget_cb (data);
+  }
   return TRUE;
 }
 
 static gboolean
 postinit_time_widget (gpointer widget)
 {
-  g_timeout_add (500, update_time_widget_cb, widget);
+  gxineinfo_private_t *priv = GET_PRIVATE (widget);
+  GtkWidget *line = GTK_WIDGET (priv->line[0]);
+  g_signal_connect (G_OBJECT (line->parent), "button-press-event",
+		    G_CALLBACK (frob_format_cb), widget);
+
+  GdkWindow *window = line->window;
+  GdkDisplay *display = gdk_drawable_get_display ((GdkDrawable *)window);
+  GdkCursor *ptr = gdk_cursor_new_for_display (display, GDK_CLOCK);
+  gdk_window_set_cursor (window, ptr);
+  gdk_cursor_unref (ptr);
+
   return FALSE;
 }
 
 GtkWidget *
-create_time_widget (gboolean small)
+create_time_widget (gboolean small, const GList *const *const listp)
 {
-  return gxineinfo_new (small ? 1 : 3, PANGO_ELLIPSIZE_NONE, small ? 16 : 8,
-			postinit_time_widget, update_time_widget_cb, "time");
+  GtkWidget *w = gxineinfo_new (small ? 1 : 3, PANGO_ELLIPSIZE_NONE,
+				small ? 16 : 8, postinit_time_widget,
+				update_time_widget_cb, "time",
+				1 /* line 0 has an event box */);
+  if (listp)
+    g_object_set_data ((GObject *) w, "gxine-chain", (gpointer) listp);
+  return w;
 }
