@@ -1,8 +1,8 @@
 /*
- * Copyright (C) 2002-2005 the xine-project
+ * Copyright (C) 2002-2006 the xine-project
  *
- * This program is free software; you can redistribute it and/or 
- * modify it under the terms of the GNU General Public License as 
+ * This program is free software; you can redistribute it and/or
+ * modify it under the terms of the GNU General Public License as
  * published by the Free Software Foundation; either version 2 of the
  * License, or (at your option) any later version.
  *
@@ -13,23 +13,21 @@
  *
  * You should have received a copy of the GNU General Public License
  * along with this program; if not, write to the Free Software
- * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA 02111-1307
+ * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
  * USA
  *
- * $Id: info_widgets.h,v 1.4 2005/06/04 15:15:09 dsalt Exp $
- *
  * nice black information display areas
  */
-#ifndef HAVE_INFO_WIDGETS_H
-#define HAVE_INFO_WIDGETS_H
+#ifndef GXINE_INFO_WIDGETS_H
+#define GXINE_INFO_WIDGETS_H
 
 #include <gtk/gtk.h>
 
 typedef GtkAlignment GtkGxineInfo;
 typedef GtkAlignmentClass GtkGxineInfoClass;
 
-GtkWidget *create_infobar (void);
-GtkWidget *create_time_widget (void);
+GtkWidget *create_infobar (gboolean small);
+GtkWidget *create_time_widget (gboolean small, const GList *const *const);
 
 void gxineinfo_update_line (GSList *, guint, const char *, ...)
   __attribute__ ((format (printf, 3, 4)));
