@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2006 the xine project
+ * Copyright (C) 2003-2007 the xine project
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -15,8 +15,6 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
  *
- * $Id: key_events.c,v 1.104 2006/04/08 21:34:49 dsalt Exp $
- *
  * key event handler and keymap editor
  */
 
@@ -57,7 +55,8 @@ static gboolean        list_visible = FALSE, edit_visible = FALSE,
 		       is_new_binding = FALSE;
 static GtkListStore   *kb_store;
 static GtkTreeModel   *kb_model;
-static GtkWidget      *kb_bindings_list_dlg, *tree_view,
+static GtkTreeView    *tree_view;
+static GtkWidget      *kb_bindings_list_dlg,
 		      *kb_binding_edit_dlg, *kb_binding_desc,
 		      *kb_binding_command, *kb_binding_key, *kb_binding_quote;
 static GtkTextBuffer  *kb_binding_command_buf;
@@ -140,14 +139,16 @@ static const key_binding_t default_bindings[] = {
   { N_("Audio"), "vdr ('AUDIO');", 0, 0 },
   /* (VDR) Info display */
   { N_("Info"), "vdr ('INFO');", 0, 0 },
+  /* (VDR) Subtitles */
+  { N_("Subtitles"), "vdr ('SUBTITLES');", 0, 0 },
   { N_("Playlist next"), "playlist_play (playlist_get_item()+1);", GDK_Page_Down, 0 },
   { N_("Playlist previous"), "playlist_play (playlist_get_item()-1);", GDK_Page_Up, 0 },
   { N_("Zoom in"), "vo_zoom.v += 5;", GDK_Z, GDK_SHIFT_MASK },
   { N_("Zoom out"), "vo_zoom.v -= 5;", GDK_z, 0 },
   { N_("Zoom 100%"), "vo_zoom.v = 100;", GDK_z, GDK_CONTROL_MASK },
-  { N_("Volume +"), "if (!vdr ('VOLPLUS'))\n{\n  ao_mute.v = false;\n  ao_volume.v += 5;\n}", 0, 0 },
-  { N_("Volume -"), "if (!vdr ('VOLMINUS'))\n{\n  ao_mute.v = false;\n  ao_volume.v -= 5;\n}", 0, 0 },
-  { N_("Mute"), "vdr ('MUTE') || ao_mute.toggle ();", 0, 0 },
+  { N_("Volume +"), "if (!vdr ('VOLPLUS'))\n{\n  ao_mute.v = false;\n  ao_volume.v += 5;\n}", GDK_v, 0 },
+  { N_("Volume -"), "if (!vdr ('VOLMINUS'))\n{\n  ao_mute.v = false;\n  ao_volume.v -= 5;\n}", GDK_V, GDK_SHIFT_MASK },
+  { N_("Mute"), "vdr ('MUTE') || ao_mute.toggle ();", GDK_m, 0 },
   { 0 }
 };
 
@@ -167,6 +168,31 @@ static key_binding_t *lookup_binding (GtkTreeIter *iter)
   return k;
 }
 
+void get_menu_bar_accel (GtkWidget *w, guint *keyval, GdkModifierType *mods)
+{
+  gchar *accelstr;
+  g_object_get (gtk_widget_get_settings (w), "gtk-menu-bar-accel",
+		&accelstr, NULL);
+  if (accelstr)
+  {
+    gtk_accelerator_parse (accelstr, keyval, mods);
+    free (accelstr);
+  }
+  if (!*keyval)
+  {
+    *keyval = GDK_F10; /* default binding */
+    *mods = GDK_SHIFT_MASK;
+  }
+}
+
+static gboolean is_menu_accel (GtkWidget *w, const GdkEventKey *event)
+{
+  guint keyval;
+  GdkModifierType mods;
+  get_menu_bar_accel (w, &keyval, &mods);
+  return event->keyval == keyval &&
+	 (event->state & GXINE_MODIFIER_MASK) == (mods & GXINE_MODIFIER_MASK);
+}  
 
 gboolean keypress_cb (GtkWidget *win, GdkEventKey *event, gpointer data)
 {
@@ -174,7 +200,7 @@ gboolean keypress_cb (GtkWidget *win, GdkEventKey *event, gpointer data)
   guint keyval = gdk_keyval_to_lower (event->keyval);
   guint state = event->state & GXINE_MODIFIER_MASK;
 
-  if (gtk_tree_model_get_iter_first (GTK_TREE_MODEL(kb_store), &iter))
+  if (gtk_tree_model_get_iter_first (kb_model, &iter))
   {
     do
     {
@@ -199,7 +225,7 @@ gboolean keypress_cb (GtkWidget *win, GdkEventKey *event, gpointer data)
        || gtk_bindings_activate_event (data, event)))
     return TRUE;
 
-  /* Windowed-mode separate toolbar (Alt-<key>, F10, Menu) */
+  /* Windowed-mode separate toolbar (Alt-<key>, GTK menu bar binding, Menu) */
   if (win == app && menubar)
   {
     GtkWidget *toplevel = gtk_widget_get_toplevel (menubar);
@@ -212,20 +238,19 @@ gboolean keypress_cb (GtkWidget *win, GdkEventKey *event, gpointer data)
     }
   }
 
-  if (state == 0)
-    switch (keyval)
-    {
-    case GDK_F10:
-      {
-	GtkWidget *toplevel = gtk_widget_get_toplevel (menubar);
-	if (menubar && GTK_WIDGET_VISIBLE (toplevel == app ? menubar : toplevel))
-	  break;
-      }
-    case GDK_Menu:
-      gtk_menu_popup (GTK_MENU(popup_menu), NULL, NULL, NULL, NULL, 0,
-                      gtk_get_current_event_time ());
-      return TRUE;
-    }
+  if (keyval == GDK_Menu && state == 0)
+  {
+    do_popup:
+    gtk_menu_popup (GTK_MENU(popup_menu), NULL, NULL, NULL, NULL, 0,
+		    gtk_get_current_event_time ());
+    return TRUE;
+  }
+  else if (is_menu_accel (win, event))
+  {
+    GtkWidget *toplevel = gtk_widget_get_toplevel (menubar);
+    if (!menubar || !GTK_WIDGET_VISIBLE (toplevel == app ? menubar : toplevel))
+      goto do_popup;
+  }
 
   logprintf ("key_events: key %d not bound\n", event->keyval);
 
@@ -292,7 +317,7 @@ static key_binding_t *find_key_binding (const gchar *desc, const gchar *cmd,
   if (!desc && !cmd)
     return NULL;
 
-  if (gtk_tree_model_get_iter_first (GTK_TREE_MODEL(kb_store), iter))
+  if (gtk_tree_model_get_iter_first (kb_model, iter))
   {
     do
     {
@@ -319,6 +344,7 @@ static gchar* kb2str (key_binding_t *kb)
   guint              l;
   gchar             *name;
   const gchar       *key;
+  const gchar	    *key_alloc = NULL;
 
   if (!text.shift)
   {
@@ -354,7 +380,7 @@ static gchar* kb2str (key_binding_t *kb)
   if (kb->keyval == GDK_VoidSymbol)
     key = _("undef"); /* undefined */
   else if (!(key = gdk_keyval_name (kb->keyval)))
-    key = _("unknown");
+    key = key_alloc = g_strdup_printf (_("unknown (%04X)"), kb->keyval);
 
   l += strlen (key);
   name = malloc (l);
@@ -377,6 +403,7 @@ static gchar* kb2str (key_binding_t *kb)
 
   strcat (name, key);
 
+  free (key_alloc);
   return name;
 }
 
@@ -476,9 +503,9 @@ void save_key_bindings (void)
   {
     GtkTreeIter iter;
 
-    fprintf (f, "<GXINEKB VERSION=\"4\">\n");
+    fprintf (f, "<GXINEKB VERSION=\"5\">\n");
 
-    if (gtk_tree_model_get_iter_first (GTK_TREE_MODEL(kb_store), &iter))
+    if (gtk_tree_model_get_iter_first (kb_model, &iter))
     {
       do
       {
@@ -573,6 +600,13 @@ static const kb_upgrade_t vdr_upgrade_to_0_5_0[] = {
   { "ao_mute.toggle ();",	"vdr ('MUTE') || ao_mute.toggle ();" },
   { NULL }
 };
+static const kb_upgrade_t upgrade_to_0_5_11[] = {
+  { "if (!vdr ('FASTREW') && !is_live_stream ()) play (0, get_time()-60000);",
+    "if (!vdr ('FASTREW') && !is_live_stream ()) { var pos = has_time() ? 0 : get_pos()-1; play (pos < 0 ? 0 : pos, get_time()-60000); }" },
+  { "if (!vdr ('FASTFWD') && !is_live_stream ()) play (0, get_time()+60000);",
+    "if (!vdr ('FASTFWD') && !is_live_stream ()) { var pos = has_time() ? 0 : get_pos()+1; play (pos < 0 ? 0 : pos, get_time()+60000); }" },
+  { NULL }
+};
 
 /* For upgrading from < 0.5.8 */
 static inline const char *
@@ -632,6 +666,8 @@ static void xml2kb (xml_node_t *node, int version)
     binding.cmd = kb_upgrade (binding.cmd, vdr_upgrade_to_0_4_0);
   if (version < 3)
     binding.cmd = kb_upgrade (binding.cmd, vdr_upgrade_to_0_5_0);
+  if (version < 5)
+    binding.cmd = kb_upgrade (binding.cmd, upgrade_to_0_5_11);
 
   logprintf ("key_events: desc='%s', cmd='%s', keyval=%d, state=%d\n",
 	     binding.desc ? binding.desc : "", binding.cmd, binding.keyval,
@@ -724,7 +760,7 @@ void kb_edit_show (void)
   {
     list_visible = TRUE;
     window_show (kb_bindings_list_dlg, NULL);
-    gtk_tree_view_columns_autosize (GTK_TREE_VIEW(tree_view));
+    gtk_tree_view_columns_autosize (tree_view);
   }
 }
 
@@ -761,14 +797,14 @@ static void kb_delete_binding (GtkWidget *widget, gpointer data)
 {
   GtkTreeIter iter;
   GtkTreeSelection *sel =
-    gtk_tree_view_get_selection (GTK_TREE_VIEW (tree_view));
+    gtk_tree_view_get_selection (tree_view);
 
   if (gtk_tree_selection_get_selected (sel, NULL, &iter))
   {
     GtkTreePath *path = gtk_tree_model_get_path (kb_model, &iter);
     if (path)
     {
-      gtk_tree_view_set_cursor (GTK_TREE_VIEW (tree_view), path, NULL, FALSE);
+      gtk_tree_view_set_cursor (tree_view, path, NULL, FALSE);
       gtk_tree_path_free (path);
     }
 
@@ -783,7 +819,7 @@ static void kb_delete_binding (GtkWidget *widget, gpointer data)
 static void kb_edit_binding (GtkWidget *widget, gpointer data)
 {
   GtkTreeSelection *sel =
-    gtk_tree_view_get_selection (GTK_TREE_VIEW (tree_view));
+    gtk_tree_view_get_selection (tree_view);
 
   if (sel && gtk_tree_selection_get_selected (sel, NULL, &catch_key_iter))
     do_edit_binding ();
@@ -895,14 +931,19 @@ static gboolean do_edit_key_cb (GtkWidget *widget, GdkEventKey *event,
 	gtk_window_activate_key (GTK_WINDOW (kb_binding_edit_dlg), event))
       return TRUE;
 
-  /* Keys not allowed */
-  if (event->state == 0)
-    switch (event->keyval)
-    {
-    case GDK_Menu: /* menu popup keys */
-    case GDK_F10:
-      return TRUE;
-    }
+  /* Keys not allowed:
+   * Shift-F10 (GTK hardwired menu popup);
+   * GTK menu bar binding (default is F10);
+   * Menu.
+   */
+  if (event->keyval == GDK_F10 &&
+      (event->state & GXINE_MODIFIER_MASK) == GDK_SHIFT_MASK)
+    return TRUE;
+  if (event->keyval == GDK_Menu && !(event->state & GXINE_MODIFIER_MASK))
+    return TRUE;
+  
+  if (is_menu_accel (widget, event))
+    return TRUE;
 
   set_binding_quote (FALSE);
 
@@ -1158,21 +1199,21 @@ static void kb_xine_event_help (se_t *se, const kb_xine_event_map_t *map)
 }
 
 
-
 static const kb_xine_event_map_t xine_vdr = {
   "VDR",
   {
-    { "AUDIO", XINE_EVENT_VDR_MUTE + 1 /* ..._VDR_AUDIO */ },
+    { "AUDIO", XINE_EVENT_VDR_AUDIO },
     { "BACK", XINE_EVENT_VDR_BACK },
     { "BLUE", XINE_EVENT_VDR_BLUE },
     { "CHANNELMINUS", XINE_EVENT_VDR_CHANNELMINUS },
     { "CHANNELPLUS", XINE_EVENT_VDR_CHANNELPLUS },
+    { "CHANNELPREV", XINE_EVENT_VDR_CHANNELPREVIOUS },
     { "CHANNELS", XINE_EVENT_VDR_CHANNELS },
     { "COMMANDS", XINE_EVENT_VDR_COMMANDS },
     { "FASTFWD", XINE_EVENT_VDR_FASTFWD },
     { "FASTREW", XINE_EVENT_VDR_FASTREW },
     { "GREEN", XINE_EVENT_VDR_GREEN },
-    { "INFO", XINE_EVENT_VDR_MUTE + 2 /* ..._VDR_INFO */ },
+    { "INFO", XINE_EVENT_VDR_INFO },
     { "MUTE", XINE_EVENT_VDR_MUTE },
     { "PAUSE", XINE_EVENT_VDR_PAUSE },
     { "PLAY", XINE_EVENT_VDR_PLAY },
@@ -1183,6 +1224,7 @@ static const kb_xine_event_map_t xine_vdr = {
     { "SCHEDULE", XINE_EVENT_VDR_SCHEDULE },
     { "SETUP", XINE_EVENT_VDR_SETUP },
     { "STOP", XINE_EVENT_VDR_STOP },
+    { "SUBTITLES", XINE_EVENT_VDR_INFO + 2 /* ..._VDR_SUBTITLES */ },
     { "TIMERS", XINE_EVENT_VDR_TIMERS },
     { "USER1", XINE_EVENT_VDR_USER1 },
     { "USER2", XINE_EVENT_VDR_USER2 },
@@ -1346,28 +1388,25 @@ void key_events_init (void)
 
   /* add a nice tree view widget here */
 
-  tree_view = gtk_tree_view_new_with_model (GTK_TREE_MODEL(kb_store));
-  gtk_tree_view_set_rules_hint (GTK_TREE_VIEW (tree_view), TRUE);
-  gtk_tree_view_set_reorderable (GTK_TREE_VIEW (tree_view), TRUE);
+  tree_view = GTK_TREE_VIEW (gtk_tree_view_new_with_model (kb_model));
+  gtk_tree_view_set_rules_hint (tree_view, TRUE);
+  gtk_tree_view_set_reorderable (tree_view, TRUE);
 
   cell = gtk_cell_renderer_text_new ();
   column = gtk_tree_view_column_new_with_attributes (_("Action"), cell,
 						     "text", COLUMN_DESC, NULL);
-  gtk_tree_view_column_set_resizable (GTK_TREE_VIEW_COLUMN(column), TRUE);
-  gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view),
-			       GTK_TREE_VIEW_COLUMN (column));
+  gtk_tree_view_column_set_resizable (column, TRUE);
+  gtk_tree_view_append_column (tree_view, column);
 
   column = gtk_tree_view_column_new_with_attributes (_("Accelerator key"), cell,
 						     "text", COLUMN_KEY, NULL);
-  gtk_tree_view_column_set_sizing (GTK_TREE_VIEW_COLUMN(column),
-				   GTK_TREE_VIEW_COLUMN_AUTOSIZE);
-  gtk_tree_view_append_column (GTK_TREE_VIEW (tree_view),
-			       GTK_TREE_VIEW_COLUMN (column));
+  gtk_tree_view_column_set_sizing (column, GTK_TREE_VIEW_COLUMN_AUTOSIZE);
+  gtk_tree_view_append_column (tree_view, column);
 
   scrolled_window = gtk_scrolled_window_new (NULL, NULL);
   gtk_scrolled_window_set_policy (GTK_SCROLLED_WINDOW (scrolled_window),
 				  GTK_POLICY_AUTOMATIC, GTK_POLICY_AUTOMATIC);
-  gtk_container_add (GTK_CONTAINER (scrolled_window), tree_view);
+  gtk_container_add (GTK_CONTAINER (scrolled_window), GTK_WIDGET (tree_view));
 
   g_signal_connect (G_OBJECT (tree_view), "row-activated",
 		    G_CALLBACK (kb_sig_edit_binding), kb_bindings_list_dlg);
@@ -1404,8 +1443,8 @@ void key_events_init (void)
 
     GError *error;
     GtkUIManager *ui = ui_create_manager ("bindings", kb_bindings_list_dlg);
-
-    gtk_action_group_add_actions (ui_get_action_group (ui),
+    GtkActionGroup *actions = ui_get_action_group (ui);
+    gtk_action_group_add_actions (actions,
 				  kb_menu_data, G_N_ELEMENTS(kb_menu_data),
 				  kb_bindings_list_dlg);
     gtk_ui_manager_add_ui_from_string (ui, kb_menu_structure, -1, &error);
@@ -1413,7 +1452,7 @@ void key_events_init (void)
       (GTK_BOX(hbox), gtk_ui_manager_get_widget (ui, "/toolbar"), TRUE, TRUE, 0);
     gtk_box_pack_end
       (GTK_BOX(hbox), gtk_ui_manager_get_widget (ui, "/menubar"), FALSE, FALSE, 0);
-    gtk_action_group_connect_accelerators (ui_get_action_group (ui));
+    gtk_action_group_connect_accelerators (actions);
   }
 
   gtk_box_pack_start (GTK_BOX(GTK_DIALOG(kb_bindings_list_dlg)->vbox), hbox,
