@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002 the xine-project
+ * Copyright (C) 2002-2006 the xine-project
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -16,12 +16,10 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
  * USA
  *
- * $Id: log_window.h,v 1.4 2006/04/08 02:51:27 dsalt Exp $
- *
  * log window header
  */
-#ifndef HAVE_LOG_WINDOW_H
-#define HAVE_LOG_WINDOW_H
+#ifndef GXINE_LOG_WINDOW_H
+#define GXINE_LOG_WINDOW_H
 
 void log_window_init (void);
 void log_window_show (void);
