@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2001-2006 the xine-project
+ * Copyright (C) 2001-2007 the xine-project
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -16,14 +16,14 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
  * USA
  *
- * $Id: main.c,v 1.163 2006/04/08 21:34:50 dsalt Exp $
- *
  * gtk2 ui for xine
  *
  * main
  */
 
 #include "globals.h"
+#include "version.h"
+#include <xine/xineutils.h>
 
 #include <string.h>
 #include <signal.h>
@@ -47,6 +47,7 @@
 #include <glib.h>
 #include <glib/gmessages.h>
 
+#include "desktop_integration.h"
 #include "engine.h"
 #include "player.h"
 #include "gtkvideo.h"
@@ -68,6 +69,7 @@
 #include "stream_info.h"
 #include "ui.h"
 #include "utils.h"
+#include "systray.h"
 #include "vis.h"
 #include "post.h"
 #include "console_output.h"
@@ -82,6 +84,13 @@ gchar          *plugindir, *bindir, *logodir, *pixmapdir, *icondir, *miscdir, *c
 gchar          *video_driver_id=NULL;
 gchar          *audio_driver_id=NULL;
 
+#define X_INIT_THREADS() \
+  if (!XInitThreads ()) \
+  { \
+    g_printerr (_("gtkvideo: XInitThreads failed - looks like you don't have a thread-safe xlib.\n")); \
+    return 2; \
+  }
+
 static void
 gxine_try_remote (int argc, char *argv[], gboolean enqueue, gboolean autoplay)
 {
@@ -142,8 +151,7 @@ static void splash_show (void)
   gtk_container_add (GTK_CONTAINER (splash), fixed);
   gtk_fixed_put (GTK_FIXED (fixed), img, 0, 0);
 
-  text = gtk_label_new ("<span font_desc='sans 12' color='#DDDDDD'>"VERSION"</span>");
-  gtk_label_set_use_markup (GTK_LABEL (text), TRUE);
+  text = ui_label_new_with_markup ("<span font_desc='sans 12' color='#DDDDDD'>"VERSION"</span>");
   gtk_misc_set_alignment (GTK_MISC (text), 1, 1);
   gtk_fixed_put (GTK_FIXED (fixed), text, 0, 0);
 
@@ -206,21 +214,20 @@ static gboolean post_init_configure (void)
 				&entry))
     vis_set (entry.enum_values[entry.num_value]);
 
+  if (xine_config_lookup_entry (xine, "gui.windowedmode_unblank", &entry))
+    gtk_video_set_windowed_unblank ((GtkVideo *)gtv, entry.num_value);
+
   return FALSE;
 }
 
 static gboolean main_start_play (gpointer data)
 {
+  if (gxine_init_count)
+    return TRUE;
   playlist_play (*(int *)data);
   return FALSE;
 }
 
-static gboolean set_fullscreen (gpointer data)
-{
-  engine_exec ("vo_fullscreen.v = true;", NULL, NULL, NULL);
-  return FALSE;
-}
-
 #ifdef WITH_WATCHDOG
 /* Watchdog functions */
 
@@ -230,7 +237,7 @@ static gboolean watchdog_timer_reset (gpointer data)
   return TRUE;
 }
 
-static void watchdog_timeout (int sig)
+static void __attribute__ ((noreturn)) watchdog_timeout (int sig)
 {
   /* if this happens, we have a problem... */
   g_printerr (_("gxine: killed by watchdog bite\n"));
@@ -238,15 +245,129 @@ static void watchdog_timeout (int sig)
 }
 #endif
 
+/* Our own locking for GDK */
+
+static GStaticRecMutex gdk_lock = G_STATIC_REC_MUTEX_INIT;
+
+static void gxine_gdk_lock (void)
+{
+  g_static_rec_mutex_lock (&gdk_lock);
+}
+
+static void gxine_gdk_unlock (void)
+{
+  g_static_rec_mutex_unlock (&gdk_lock);
+}
+
+/* Faults */
+
+static void __attribute__ ((noreturn)) fault_handler (int sig)
+{
+  static gboolean recurse = FALSE;
+  if (recurse)
+    abort ();
+
+  signal (sig, fault_handler);
+  recurse = TRUE;
+
+  const char *sigtext = strsignal (sig);
+  fputs (_("gxine has suffered a fatal internal error.\n"
+	   "To get a backtrace, run gxine in a debugger such as gdb.\n"
+	   "Then, when the error occurs:\n"
+	   "  (gdb) thread apply all bt\n"), stderr);
+  display_error_modal (FROM_GXINE, _("Fatal error"), "%s",
+		       sigtext ? : _("Unknown"));
+  exit (2);
+}
+
 /* Main */
 
+static char *
+getdir (const char *env, const char *dir)
+{
+  return getenv (env) ? : g_strconcat (dir, NULL);
+}
+
+static void
+list_plugins (const char *msg, typeof (xine_list_audio_output_plugins) list_func)
+{
+  static xine_t *xine = NULL;
+  if (!xine)
+    xine_init (xine = xine_new ());
+  g_print ("%s", msg);
+  const char *const *list = list_func (xine);
+  while (list && *list)
+  {
+    g_print (" %s", *list);
+    ++list;
+  }
+  g_print ("\n");
+}
+
+static void config_change (gpointer data, gpointer user)
+{
+  xine_cfg_entry_t entry;
+  int i;
+  char *text = data;
+  char *equals = strchr (text, '=');
+  *equals = 0;
+
+  if (xine_config_lookup_entry (xine, text, &entry))
+    switch (entry.type)
+    {
+      case XINE_CONFIG_TYPE_RANGE:
+	i = atoi (equals + 1);
+	entry.num_value = MIN (entry.range_max, MAX (entry.range_min, i));
+	xine_config_update_entry (xine, &entry);
+	break;
+
+      case XINE_CONFIG_TYPE_NUM:
+	entry.num_value = atoi (equals + 1);
+	xine_config_update_entry (xine, &entry);
+	break;
+
+      case XINE_CONFIG_TYPE_BOOL:
+	entry.num_value = !!atoi (equals + 1);
+	xine_config_update_entry (xine, &entry);
+	break;
+
+      case XINE_CONFIG_TYPE_STRING:
+	entry.str_value = equals + 1;
+	xine_config_update_entry (xine, &entry);
+	break;
+
+      case XINE_CONFIG_TYPE_ENUM:
+	for (i = 0; entry.enum_values[i]; ++i)
+	  if (!strcmp (equals + 1, entry.enum_values[i]))
+	    break;
+	if (entry.enum_values[i])
+	{
+	  entry.num_value = i;
+	  xine_config_update_entry (xine, &entry);
+	}
+	else /* given value isn't in the list of valid values */
+	  g_printerr (_("gxine: warning: bad value for config item '%s'\n"), text);
+	break;
+    }
+  else
+    g_printerr (_("gxine: warning: config item '%s' not found\n"), text);
+
+  *equals = '=';
+}  
+
 int main(int argc, char* argv[])
 {
   gboolean enqueue, autoplay, fullscreen;
-  gboolean no_connect;
+  gboolean no_connect, debug;
   int optstate;
   int show_splash = 2; /* default */
   const char *cmd = NULL;
+  char *geometry = NULL;
+  xine_cfg_entry_t entry;
+  GPtrArray *cfg_changes;
+  gboolean migrated_config_dir = FALSE;
+
+  g_thread_init (NULL); /* done early for glib >= 2.13 */
 
 #ifndef GLIB_USES_SYSTEM_MALLOC
   GMemVTable vtab = { xmalloc, xrealloc, free, NULL, NULL, NULL };
@@ -261,25 +382,22 @@ int main(int argc, char* argv[])
    * be overwritten by environment variables
    */
 
-  plugindir = getenv("GXINE_PLUGINDIR")	? : g_strconcat(GXINE_PLUGINDIR, NULL);
-  bindir    = getenv("GXINE_BINDIR")	? : g_strconcat(GXINE_BINDIR, NULL);
-  logodir   = getenv("GXINE_LOGODIR")	? : g_strconcat(GXINE_LOGODIR, NULL);
-  pixmapdir = getenv("GXINE_PIXMAPDIR")	? : g_strconcat(GXINE_PIXMAPDIR, NULL);
-  icondir   = getenv("GXINE_ICONDIR")	? : g_strconcat(GXINE_ICONDIR, NULL);
-  miscdir   = getenv("GXINE_MISCDIR")	? : g_strconcat(GXINE_MISCDIR, NULL);
-  confdir   = getenv("GXINE_CONFDIR")	? : g_strconcat(GXINE_CONFDIR, NULL);
+  plugindir = getdir ("GXINE_PLUGINDIR", GXINE_PLUGINDIR);
+  bindir    = getdir ("GXINE_BINDIR", GXINE_BINDIR);
+  logodir   = getdir ("GXINE_LOGODIR", GXINE_LOGODIR);
+  pixmapdir = getdir ("GXINE_PIXMAPDIR", GXINE_PIXMAPDIR);
+  icondir   = getdir ("GXINE_ICONDIR", GXINE_ICONDIR);
+  miscdir   = getdir ("GXINE_MISCDIR", GXINE_MISCDIR);
+  confdir   = getdir ("GXINE_CONFDIR", GXINE_CONFDIR);
 
   /* set up our gtkrc stuff */
   {
     gchar **file = gtk_rc_get_default_files ();
-    gchar **copy;
     const gchar *home = g_get_home_dir ();
     guint homelen = strlen (home);
-    guint length = 0, i;
-    while (file[length])
-      ++length;
+    guint i, length = g_strv_length (file);
     /* deep-copy the gtkrc filenames, allocating space for two more */
-    copy = calloc (length + 3, sizeof (gchar *));
+    gchar **copy = calloc (length + 3, sizeof (gchar *));
     for (i = 0; file[i]; ++i)
       copy[i] = strdup (file[i]);
     /* find the index of the first *user* gtkrc filename */
@@ -294,9 +412,7 @@ int main(int argc, char* argv[])
     copy[length + 2] = NULL;
     gtk_rc_set_default_files (copy);
     /* free the list */
-    for (i = 0; copy[i]; ++i)
-      free (copy[i]);
-    free (copy);
+    g_strfreev (copy);
   }
 
   /*
@@ -316,10 +432,13 @@ int main(int argc, char* argv[])
   textdomain (PACKAGE);
 #endif
 
-  g_thread_init (NULL);
   console_output_init ();
+  gdk_threads_set_lock_functions (gxine_gdk_lock, gxine_gdk_unlock);
   gdk_threads_init ();
 
+  if (!gtk_parse_args (&argc, &argv))
+    return 2;
+
   /*
    * parse command line arguments
    */
@@ -329,12 +448,14 @@ int main(int argc, char* argv[])
   autoplay = TRUE;
   fullscreen = FALSE;
   no_connect = FALSE;
+  debug = FALSE;
   optstate = 0;
+  cfg_changes = g_ptr_array_new ();
   for (;;)
   {
-#define OPTS "hvaefV:A:sSc:n"
+#define OPTS "hvaefV:A:sSc:C:n"
 #ifdef HAVE_GETOPT_LONG
-    static struct option longopts[] = {
+    static const struct option longopts[] = {
       { "help", no_argument, NULL, 'h' },
       { "version", no_argument, NULL, 1 },
       { "verbose", no_argument, NULL, 'v' },
@@ -346,6 +467,9 @@ int main(int argc, char* argv[])
       { "splash", no_argument, NULL, 's' },
       { "no-splash", no_argument, NULL, 'S' },
       { "command", required_argument, NULL, 'c' },
+      { "configure", required_argument, NULL, 'C' },
+      { "geometry", required_argument, NULL, 2 },
+      { "debug", no_argument, NULL, 3 },
       { NULL }
     };
     int index = 0;
@@ -364,6 +488,12 @@ int main(int argc, char* argv[])
     case 1:
       optstate |= 4;
       break;
+    case 2:
+      geometry = optarg;
+      break;
+    case 3:
+      debug = TRUE;
+      break;
     case 'v':
       verbosity++;
       break;
@@ -396,6 +526,12 @@ int main(int argc, char* argv[])
     case 'n':
       no_connect = TRUE;
       break;
+    case 'C':
+      if (strchr (optarg, '='))
+	g_ptr_array_add (cfg_changes, optarg);
+      else
+	g_printerr (_("gxine: warning: bad parameter for --configure: '%s'\n"), optarg);
+      break;
     default:
       optstate |= 2;
       break;
@@ -403,6 +539,7 @@ int main(int argc, char* argv[])
   }
 
   if (optstate & 1)
+  {
     g_print (_("\
 gxine %s (%s)\n\
 usage: %s [options] [MRLs...]\n\
@@ -416,14 +553,20 @@ options:\n\
   -e, --enqueue		don't clear the playlist, don't play the new items\n\
   -f, --full-screen	start in full-screen mode\n\
   -v, --verbose		be more verbose\n\
-\n"), VERSION, VENDOR_PKG_VERSION, argv[0]);
+  -C, --configure KEY=VALUE\n\
+                        alter a configuration item\n\
+\n"), VERSION, VENDOR_PKG_VERSION CSET_ID, argv[0]);
+    list_plugins (_("Available video drivers:"), xine_list_video_output_plugins);
+    list_plugins (_("Available audio drivers:"), xine_list_audio_output_plugins);
+  }
   else if (optstate & 4)
-    g_print ("gxine %s (%s)\n%s\n%s", VERSION, VENDOR_PKG_VERSION,
-	    _("© 2002-2006 the xine project team"), _("\
+    g_print (_("gxine %s (%s)\nusing xine-lib %s, compiled using %s\n%s\n\
 This is free software; see the source for copying conditions.  There is NO\n\
 warranty; not even for MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE,\n\
-to the extent permitted by law.\n"));
-
+to the extent permitted by law.\n"),
+	     VERSION, VENDOR_PKG_VERSION CSET_ID,
+	     xine_get_version_string (), XINE_VERSION,
+	     get_copyright_notice ());
   if (optstate & 2)
   {
     g_printerr (_("gxine: invalid option (try -h or --help)\n"));
@@ -438,6 +581,9 @@ to the extent permitted by law.\n"));
   gtk_init(&argc, &argv);
   gtk_window_set_auto_startup_notification (FALSE);
 
+  if (!debug)
+    signal (SIGSEGV, fault_handler);
+
   /*
    * find out if gxine is already running, if so
    * just pass on the files to play
@@ -447,26 +593,43 @@ to the extent permitted by law.\n"));
   if (!no_connect)
     gxine_try_remote (argc, argv, enqueue, autoplay);
 
+#if !defined(HAVE_XCB) && !defined(USE_SUN_XLIB_WORKAROUND)
   /* start using X... */
-  if (!XInitThreads ())
-  {
-    g_printerr (_("gtkvideo: XInitThreads failed - looks like you don't have a thread-safe xlib.\n"));
-    return 2;
-  }
+  X_INIT_THREADS ();
+#endif
+
+  struct stat st;
+  char *oldconfig = g_build_filename (g_get_home_dir (), ".gxine", NULL);
+  char *newconfig = get_config_filename (NULL);
 
   /*
-   * make sure ~/.gxine exists
+   * make sure $XDG_CONFIG_DIR/gxine exists, migrating from ~/.gxine if needed
    */
+  if (strcmp (oldconfig, newconfig) &&
+      !stat (oldconfig, &st) && stat (newconfig, &st) && errno == ENOENT)
   {
-    char *fname = get_config_filename (NULL);
-    int r = ensure_path_exists (fname, 0700);
+    char *tail = strrchr (newconfig, '/');
+    *tail = 0; /* chop off the leafname for mkdir */
+    int r = ensure_path_exists (newconfig, 0700);
+    *tail = '/'; /* restore for the rename */
+    if (r || rename (oldconfig, newconfig))
+      display_warning
+	(FROM_GXINE, _("File creation error"),
+	 _("Couldn't create %s: %s.\n"
+	   "Configuration, playlist and media marks will not be saved."),
+	 newconfig, strerror (r ? r : errno));
+    else
+      migrated_config_dir = TRUE;
+  }
+  else
+  {
+    int r = ensure_path_exists (newconfig, 0700);
     if (r)
       display_warning
 	(FROM_GXINE, _("File creation error"),
-	 _("Couldn't create ~/.gxine: %s.\n"
+	 _("Couldn't create %s: %s.\n"
 	   "Configuration, playlist and media marks will not be saved."),
-	 strerror (r));
-    g_free (fname);
+	 newconfig, strerror (r));
   }
 
 #ifdef WITH_WATCHDOG
@@ -474,8 +637,11 @@ to the extent permitted by law.\n"));
    * Watchdog
    */
 
-  g_timeout_add (2500, watchdog_timer_reset, NULL); /* <- will set the alarm */
-  signal (SIGALRM, watchdog_timeout);
+  if (!debug)
+  {
+    g_timeout_add (2500, watchdog_timer_reset, NULL); /* <- will set the alarm */
+    signal (SIGALRM, watchdog_timeout);
+  }
 #endif
 
   /*
@@ -485,16 +651,25 @@ to the extent permitted by law.\n"));
   engine_init ();
   server_setup ();
 
+  static const xine_config_entry_translation_t gxine_config_xlate[] = {
+    { "gui.windowwindow_size", "gui.windowed_mode.window_size" },
+    { "gui.magnify_lowres_video", "gui.windowed_mode.magnify_lowres_video" },
+    { "gui.auto_resize", "gui.windowed_mode.auto_resize" },
+    { "gui.fullscreen_always_sticky", "gui.fullscreen_mode.always_sticky" },
+    { "gui.fullscreen_toolbar", "gui.fullscreen_mode.toolbar" },
+    { "gui.windowedmode_separate_toolbar", "gui.windowed_mode.separate_toolbar" },
+    { "gui.windowedmode_unblank", "gui.windowed_mode.unblank" },
+    {}
+  };
+  xine_config_set_translation_user (gxine_config_xlate);
+
   /*
    * a splash screen for the impatient
    */
 
   if (show_splash == 2)
-  {
-    xine_cfg_entry_t entry;
     show_splash = !xine_config_lookup_entry (xine, "gui.show_splash", &entry)
 		  || entry.num_value;
-  }
 
   if (show_splash)
     splash_show ();
@@ -502,6 +677,37 @@ to the extent permitted by law.\n"));
   gdk_threads_enter ();
   do_pending_events ();
 
+  /*
+   * finish configuration migration (if needed)
+   */
+  if (migrated_config_dir && xine_config_get_first_entry (xine, &entry))
+  {
+    int oldlen = strlen (oldconfig);
+  
+    do
+    {
+      if (entry.type == XINE_CONFIG_TYPE_STRING &&
+          (!xine_check_version (1, 1, 4) || entry.num_value) &&
+          !strncmp (entry.str_value, oldconfig, oldlen) &&
+          (entry.str_value[oldlen] == '/' || entry.str_value[oldlen] == 0))
+      {
+	entry.str_value = g_strconcat (newconfig, entry.str_value + oldlen, NULL);
+        logprintf (_("updating config item %s to %s\n"), entry.key, entry.str_value);
+	xine_config_update_entry (xine, &entry);
+	g_free (entry.str_value);
+      }
+    } while (xine_config_get_next_entry (xine, &entry));
+  }
+
+  if (migrated_config_dir)
+    display_info (FROM_GXINE, _("gxine configuration moved"),
+		  _("gxine's configuration has been moved to a new location, "
+		    "in accordance with the freedesktop.org Desktop Entry "
+		    "Specification."));
+
+  g_free (oldconfig);
+  g_free (newconfig);
+
   /*
    * set up player and some UI basics
    */
@@ -520,7 +726,12 @@ to the extent permitted by law.\n"));
 
   do_pending_events ();
   gtk_window_set_auto_startup_notification (TRUE);
-  noskin_main_init ();
+  noskin_main_init (geometry, fullscreen);
+
+#if !defined(HAVE_XCB) && defined(USE_SUN_XLIB_WORKAROUND)
+  /* call XInitThreads after the window opens to avoid deadlock in libXi */
+  X_INIT_THREADS ();
+#endif
 
   do_pending_events ();
 
@@ -538,12 +749,16 @@ to the extent permitted by law.\n"));
   mediamarks_init  ();
   play_item_init   ();
   key_events_init  ();
-  gxine_lirc_init  ();
   snapshot_init    ();
   vis_init	   ();
+  systray_init	   ();
   post_init	   ();
   stream_info_init ();
+  gxine_vfs_init   ();
   wizards_init     ();
+  gxine_lirc_init  ();
+
+  g_ptr_array_foreach (cfg_changes, config_change, NULL);
 
   engine_startup_script ();
 
@@ -557,6 +772,9 @@ to the extent permitted by law.\n"));
   post_init_configure ();
   initialised = TRUE;
 
+  gxine_init_add ((GtkFunction) ui_post_init, NULL);
+  playlist_logo ((gpointer) 1);
+
   /*
    * argument parsing
    */
@@ -575,18 +793,10 @@ to the extent permitted by law.\n"));
       else
 	playlist_add_mrl (argv[i], -1);
 
-    if (first < 0)
-      gtk_init_add ((GtkFunction) playlist_logo, NULL);
-    else
-      gtk_init_add (main_start_play,  &first);
+    if (first >= 0)
+      g_idle_add (main_start_play, &first);
   }
-  else
-    gtk_init_add ((GtkFunction) playlist_logo, NULL);
 
-  if (fullscreen)
-    gtk_init_add (set_fullscreen, NULL);
-
-  gtk_init_add ((GtkFunction) ui_post_init, NULL);
   server_start ();
 
   /* execute any command-line JS */
