@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2003-2005 the xine project
+ * Copyright (C) 2003-2007 the xine project
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -15,8 +15,6 @@
  * along with this program; if not, write to the Free Software
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301 USA
  *
- * $Id: menu.c,v 1.82 2006/04/08 21:34:50 dsalt Exp $
- *
  * main/context menu creation / handling
  */
 
@@ -92,7 +90,7 @@ static void autoplay_cb (GtkAction *action, const char *data)
       }
     }
 
-    play_item_t *item = play_item_new (NULL, mrls[i], 0);
+    play_item_t *item = play_item_new (NULL, mrls[i], 0, 0);
     item->type = PLAY_ITEM_AUTOPLAY;
     if (pos < 0)
       pos = playlist_add (item, -1);
@@ -304,6 +302,7 @@ JS_CB_T (WMUnblank)
 {
   gtk_video_set_windowed_unblank ((GtkVideo *)gtv, v);
 }
+JS_CB_T (FSSticky) { window_fs_stickiness_update (v); }
 
 /* Audio */
 JS_CB (AudioSettings, "settings_show (1);")
@@ -358,16 +357,17 @@ static const GtkActionEntry main_actions[] = {
   CB(Zoom100,		GTK_STOCK_ZOOM_100,	N_("_Zoom 100%"),		NULL),
   { "AspectMenu",	NULL,			N_("_Aspect ratio"),		NULL },
   { "VisMenu",		NULL,			N_("_Visualisations"),		NULL },
-  { "SubsMenu",		NULL,			N_("_Subtitles"),		NULL },
+  { "SubsMenu",		"gxine-settings-spu",	N_("_Subtitles"),		NULL },
+  { "ViewMisc",		GTK_STOCK_PROPERTIES,	N_("_Settings"),		NULL },
   { "VideoMenu",	NULL,			N_("V_ideo"),			NULL },
-  CB(VideoSettings,	GTK_STOCK_PROPERTIES,	/* Video */ N_("_Settings..."),	NULL),
-  { "VideoConfigMenu",	GTK_STOCK_PREFERENCES,	N_("_Configure"),		NULL },
+  CB(VideoSettings,	GTK_STOCK_PROPERTIES,	/* Video */ N_("_Controls..."),	NULL),
+  { "VideoConfigMenu",	GTK_STOCK_PREFERENCES,	N_("_Configure plugins"),	NULL },
   CB(DeinterlaceConf,	NULL,			N_("_Deinterlace..."),		NULL),
   CB(VideoPPConf,	NULL,			N_("_Video post-processing..."),NULL),
   { "AudioMenu",	NULL,			N_("_Audio"),			NULL },
-  CB(AudioSettings,	GTK_STOCK_PROPERTIES,	/* Audio */ N_("_Settings..."),	NULL),
+  CB(AudioSettings,	GTK_STOCK_PROPERTIES,	/* Audio */ N_("_Controls..."),	NULL),
   CB(AudioEq,		GTK_STOCK_PROPERTIES,	N_("_Equaliser..."),		NULL),
-  { "AudioConfigMenu",	GTK_STOCK_PREFERENCES,	N_("_Configure"),		NULL },
+  { "AudioConfigMenu",	GTK_STOCK_PREFERENCES,	N_("_Configure plugins"),		NULL },
   CB(AudioPPConf,	NULL,			N_("_Audio post-processing..."),NULL),
   { "MediaMenu",	NULL,			N_("_Media"),			NULL },
   CB(MediaAdd,		GTK_STOCK_NEW,		N_("_Add media mark..."),	"<control>D"),
@@ -376,7 +376,7 @@ static const GtkActionEntry main_actions[] = {
   { "ToolsMenu",	NULL,			N_("_Tools"),			NULL },
   CB(EngineLog,		NULL,			N_("Engine _log..."),		"<control>L"),
   CB(JSConsole,		GTK_STOCK_EXECUTE,	N_("_Javascript console..."),	"<control>J"),
-  CB(StreamInfo,	"gtk-info",		N_("Stream _info..."),		NULL),
+  CB(StreamInfo,	GTK_STOCK_INFO,		N_("Stream _info..."),		NULL),
   CB(SetupWizards,	GTK_STOCK_REDO,		N_("_Re-run setup wizards..."), NULL),
   { "HelpMenu",		NULL,			N_("_Help"),			NULL },
   CB(About,		GTK_STOCK_ABOUT,	N_("_About..."),		NULL),
@@ -407,13 +407,14 @@ static GtkToggleActionEntry toggle_actions[] = {
   CB(Deinterlace,	NULL, N_("_Deinterlace"),		"<control>I",	FALSE),
   CB(VideoPP,		NULL, N_("_Post-processing"),		NULL,		FALSE),
   CB(AudioPP,		NULL, N_("_Post-processing"),		NULL,		FALSE),
-  CB(WMUnblank,		NULL, N_("Prevent _blanking\nin windowed mode"), NULL,	FALSE),
+  CB(WMUnblank,		NULL, N_("Prevent _blanking in windowed mode"), NULL,	FALSE),
+  CB(FSSticky,		NULL, N_("Force _stickiness in full-screen mode"), NULL,	FALSE),
 };
 static const char *const toggle_prefs[] = {
-  NULL, NULL, NULL, NULL, NULL,
-  "gui.magnify_lowres_video", "gui.post_plugins.deinterlace_enable",
+  NULL, NULL, NULL, NULL, "gui.windowed_mode.auto_resize",
+  "gui.windowed_mode.magnify_lowres_video", "gui.post_plugins.deinterlace_enable",
   "gui.post_plugins.video_enable", "gui.post_plugins.audio_enable",
-  "gui.windowedmode_unblank"
+  "gui.windowed_mode.unblank", "gui.fullscreen_mode.always_sticky",
 };
 #undef CB
 
@@ -443,10 +444,7 @@ vis_actions[] = {
   { "VisNone",		NULL, N_("_None"),	NULL, NULL, 0 },
 },
 subtitle_actions[] = {
-  { "SubtitlesOther",	NULL, "",		NULL, NULL, -3 },
-  /* Subtitles menu text */
-  { "SubtitlesNone",	NULL, N_("_None"),	NULL, NULL, -2 },
-  { "SubtitlesAuto",	NULL, N_("_Auto"),	NULL, NULL, -1 },
+  /* menu item order is relied upon in player.c:set_subtitles() */
   { "Subtitles0",	NULL, N_("Channel _0"),	NULL, NULL, 0 },
   { "Subtitles1",	NULL, N_("Channel _1"),	NULL, NULL, 1 },
   { "Subtitles2",	NULL, N_("Channel _2"),	NULL, NULL, 2 },
@@ -463,6 +461,11 @@ subtitle_actions[] = {
   { "Subtitles13",	NULL, N_("Channel 13"),	NULL, NULL, 13 },
   { "Subtitles14",	NULL, N_("Channel 14"),	NULL, NULL, 14 },
   { "Subtitles15",	NULL, N_("Channel 15"),	NULL, NULL, 15 },
+  /* Subtitles menu text */
+  { "SubtitlesNone",	NULL, N_("_None"),	NULL, NULL, -2 },
+  { "SubtitlesAuto",	NULL, N_("_Auto"),	NULL, NULL, -1 },
+  /* Other subtitle channels */
+  { "SubtitlesOther",	NULL, "",		NULL, NULL, -3 },
 };
 
 static GtkWidget *add_menu_ui (GtkUIManager *ui, int i)
@@ -534,7 +537,6 @@ static GtkWidget *add_menu_ui (GtkUIManager *ui, int i)
 	    "<menuitem action='VisNone' />\n"
 	    "<placeholder name='VisItems' />\n"
 	  "</menu>\n"
-	  "<separator />\n"
 	  "<menu action='SubsMenu'>\n"
 	    "<menuitem action='SubtitlesAuto' />\n"
 	    "<menuitem action='SubtitlesNone' />\n"
@@ -555,6 +557,10 @@ static GtkWidget *add_menu_ui (GtkUIManager *ui, int i)
 	    "<menuitem action='Subtitles14' />\n"
 	    "<menuitem action='Subtitles15' />\n"
 	  "</menu>\n"
+	  "<menu action='ViewMisc'>\n"
+	    "<menuitem action='WMUnblank' />\n"
+	    "<menuitem action='FSSticky' />\n"
+          "</menu>\n"
 	  "<separator />\n"
 	  "<menuitem action='EngineLog' />\n"
 	  "<menuitem action='JSConsole' />\n"
@@ -568,7 +574,6 @@ static GtkWidget *add_menu_ui (GtkUIManager *ui, int i)
 	    "<menuitem action='DeinterlaceConf' />\n"
 	    "<menuitem action='VideoPPConf' />\n"
 	  "</menu>\n"
-	  "<menuitem action='WMUnblank' />\n"
 	"</menu>\n"
 	"<menu action='AudioMenu'>\n"
 	  "<menuitem action='AudioSettings' />\n"
@@ -657,6 +662,45 @@ void scale_changed_cb (GtkWidget *widget, gdouble factor, gpointer data)
     logprintf ("gxine: ignored scale change event (%lf%%)\n", factor);
 }
 
+static void update_subtitles_menu_internal (GtkActionGroup *action)
+{
+  if (!action)
+    return;
+  int i;
+  for (i = 0; i < 16; ++i)
+  {
+    static char subt[] = "Subtitles..";
+    char lang[XINE_LANG_MAX];
+    sprintf (subt + 9, "%d", i);
+    GtkAction *item = get_action (subt);
+    if (!item)
+      break;
+    char *label = NULL, *base = gettext (subtitle_actions[i].label);
+    if (xine_get_spu_lang (stream, i, lang))
+    {
+      label = base - 1;
+      while (*++label == ' ')
+        /* skip spaces */;
+      label = g_strconcat (label, strcmp (lang, "none") ? " / " : NULL, lang, NULL);
+    }
+    g_object_set ((GObject *) item, "label", label ? label : base, NULL);
+    free (label);
+  }
+}
+
+static gboolean subs_menu_cb (GtkWidget *widget, GdkEventExpose *event,
+			      gpointer action)
+{
+  update_subtitles_menu_internal (action);
+  return FALSE;
+}
+
+void update_subtitles_menu (void)
+{
+  if (ui)
+    update_subtitles_menu_internal (ui_get_action_group (ui));
+}
+
 void create_menus (GtkWidget *window)
 {
   GtkActionGroup *action;
@@ -673,7 +717,7 @@ void create_menus (GtkWidget *window)
 	xine_config_lookup_entry (xine, toggle_prefs[i], &entry))
       toggle_actions[i].is_active = entry.num_value;
 
-  if (!xine_config_lookup_entry (xine, "gui.fullscreen_toolbar", &entry))
+  if (!xine_config_lookup_entry (xine, "gui.fullscreen_mode.toolbar", &entry))
     entry.num_value = 0;
   toggle_actions[TOGGLE_FULLSCREEN_TOOLBAR].is_active = (entry.num_value >> 1) & 1;
 
@@ -719,6 +763,7 @@ void create_menus (GtkWidget *window)
   action_items.wm_toolbar = get_toggle ("WMToolbarVisible");
   action_items.wm_toolbar_snap = get_toggle ("WMToolbarSnap");
   action_items.wm_unblank = get_toggle ("WMUnblank");
+  action_items.fs_sticky = get_toggle ("FSSticky");
 
   action_items.aspect = get_radio_group ("AspectAuto");
   action_items.vis = get_radio_group ("VisNone");
@@ -728,11 +773,18 @@ void create_menus (GtkWidget *window)
 
   action_items.keypad = get_action ("Keypad");
 
-  if (xine_config_lookup_entry (xine, "gui.windowedmode_separate_toolbar",
+  if (xine_config_lookup_entry (xine, "gui.windowed_mode.separate_toolbar",
 				&entry)
       && !entry.num_value)
   {
     gtk_action_set_sensitive (&action_items.wm_toolbar->parent, FALSE);
     gtk_action_set_sensitive (&action_items.wm_toolbar_snap->parent, FALSE);
   }
+
+  GSList *proxy = gtk_action_get_proxies (get_action ("SubsMenu"));
+  while (proxy)
+  {
+    g_signal_connect (proxy->data, "event", subs_menu_cb, action);
+    proxy = g_slist_next (proxy);
+  }
 }
