@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2006 the xine-project
+ * Copyright (C) 2002-2007 the xine-project
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -16,8 +16,6 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
  * USA
  *
- * $Id: playlist.c,v 1.175 2006/04/08 21:34:50 dsalt Exp $
- *
  * playlist implementation
  *
  * .pls parser stolen from totem (C) 2002 Bastien Nocera
@@ -40,6 +38,7 @@
 
 #include <gtk/gtk.h>
 #include <gdk/gdk.h>
+#include <gdk/gdkkeysyms.h>
 #include <glib.h>
 #include <xine/xmlparser.h>
 
@@ -50,6 +49,7 @@
 #include "ui.h"
 #include "utils.h"
 #include "menu.h"
+#include "list_menus.h"
 #include "play_item.h"
 #include "mediamarks.h"
 #include "engine.h"
@@ -63,13 +63,20 @@
 static int             is_visible;
 static GtkUIManager   *pl_ui;
 static GtkListStore   *pl_store;
-static GtkWidget      *dlg;
+static GtkTreeModel   *pl_model;
+static GtkWidget      *dlg, *menu[2];
 static GtkTreeView    *tree_view;
 static GtkTreeSelection *sel;
-static GtkToggleButton *repeat_button, *random_button;
+static GtkToggleButton *repeat_button, *random_button, *repeat_track_button;
 static char            logo_mrl[1024];
 static int             logo_mode;
 
+static struct {
+  GtkDialog *dbox;
+  GtkLabel *label;
+  GtkEntry *name, *pass;
+} auth;
+
 static play_item_t    *cur_item = NULL;
 static int             cur_list_pos;
 static int	       ref_list_pos;
@@ -93,8 +100,7 @@ static int playlist_clip (int);
 static play_item_t *peek_play_item (GtkTreeIter *iter)
 {
   GValue v = {};
-  gtk_tree_model_get_value (GTK_TREE_MODEL (pl_store), iter,
-			    COLUMN_PLAY_ITEM, &v);
+  gtk_tree_model_get_value (pl_model, iter, COLUMN_PLAY_ITEM, &v);
   void *p = g_value_peek_pointer (&v);
   g_value_unset (&v);
   return p;
@@ -121,8 +127,7 @@ static inline void cur_item_dispose (void)
 static gboolean item_marked_current (GtkTreeIter *iter)
 {
   GValue v = {};
-  gtk_tree_model_get_value (GTK_TREE_MODEL (pl_store), iter,
-			    COLUMN_MARKING, &v);
+  gtk_tree_model_get_value (pl_model, iter, COLUMN_MARKING, &v);
   const char *p = g_value_peek_pointer (&v);
   gboolean ret = p && *p;
   g_value_unset (&v);
@@ -163,7 +168,7 @@ static gboolean playlist_browse_set (const play_item_t *item)
 
   for (i = 0; i < count; ++i)
   {
-    play_item_t *item = play_item_new (NULL, mrls[i]->mrl, 0);
+    play_item_t *item = play_item_new (NULL, mrls[i]->mrl, 0, 0);
     item->type = PLAY_ITEM_BROWSER;
     playlist_add (item, -1);
   }
@@ -171,6 +176,13 @@ static gboolean playlist_browse_set (const play_item_t *item)
   return FALSE;
 }
 
+static gboolean map_cb (GtkWidget* widget, gpointer data)
+{
+  gdk_window_set_events (widget->window,
+			 gdk_window_get_events (widget->window) | GDK_BUTTON_PRESS_MASK);
+  return FALSE;
+}
+
 static gboolean close_cb (GtkWidget* widget, gpointer data)
 {
   is_visible = FALSE;
@@ -185,7 +197,7 @@ int playlist_flush (play_item_type type)
   int count = 0;
 
   restart:
-  if (!gtk_tree_model_get_iter_first (GTK_TREE_MODEL (pl_store), &iter))
+  if (!gtk_tree_model_get_iter_first (pl_model, &iter))
     goto done;
 
   int pos = 0;
@@ -203,7 +215,7 @@ int playlist_flush (play_item_type type)
     }
 
     ++pos;
-  } while (gtk_tree_model_iter_next (GTK_TREE_MODEL(pl_store), &iter));
+  } while (gtk_tree_model_iter_next (pl_model, &iter));
 
   done:
   return count;
@@ -241,7 +253,7 @@ static void del_cb (GtkWidget* widget, gpointer data)
     GtkTreePath *path;
     gint         pos;
 
-    path = gtk_tree_model_get_path (GTK_TREE_MODEL (pl_store), &iter);
+    path = gtk_tree_model_get_path (pl_model, &iter);
     indices = gtk_tree_path_get_indices (path);
     pos = indices[0];
 
@@ -278,7 +290,7 @@ static void paste_cb (GtkWidget* widget, gpointer data)
     if (gtk_tree_selection_get_selected (sel, NULL, &iter))
     {
       GtkTreePath *path;
-      path = gtk_tree_model_get_path (GTK_TREE_MODEL (pl_store), &iter);
+      path = gtk_tree_model_get_path (pl_model, &iter);
       playlist_add (item, gtk_tree_path_get_indices (path)[0] + 1);
       gtk_tree_path_free (path);
     }
@@ -289,7 +301,7 @@ static void paste_cb (GtkWidget* widget, gpointer data)
 
 static void new_cb (GtkWidget* widget, gpointer data)
 {
-  play_item_t *play_item = play_item_new ("", "", 0);
+  play_item_t *play_item = play_item_new ("", "", 0, 0);
   if (play_item_edit (play_item, PLAY_ITEM_LIST_NEW, NULL, dlg))
     playlist_add (play_item, -1);
   else
@@ -323,6 +335,18 @@ static void add_cb (GtkWidget* widget, gpointer data)
      FALSE);
 }
 
+static void play_cb (GtkWidget* widget, gpointer data)
+{
+  GtkTreeIter iter;
+
+  if (gtk_tree_selection_get_selected (sel, NULL, &iter))
+  {
+    GtkTreePath *path = gtk_tree_model_get_path (pl_model, &iter);
+    playlist_play (gtk_tree_path_get_indices (path)[0]);
+    gtk_tree_path_free (path);
+  }
+}
+
 static void edit_cb (GtkWidget* widget, gpointer data)
 {
   GtkTreeIter iter;
@@ -407,7 +431,7 @@ static int playlist_load (const char *fname)
     goto ret0;
   }
 
-  xml_parser_init (plfile, strlen (plfile), XML_PARSER_CASE_INSENSITIVE);
+  xml_parser_init (plfile, st.st_size, XML_PARSER_CASE_INSENSITIVE);
 
   if (xml_parser_build_tree (&root) < 0)
   {
@@ -451,6 +475,8 @@ static int playlist_load (const char *fname)
 	  (random_button, xml_parser_get_property_bool (node, "RANDOM", 0));
 	gtk_toggle_button_set_active
 	  (repeat_button, xml_parser_get_property_bool (node, "REPEAT", 0));
+	gtk_toggle_button_set_active
+	  (repeat_track_button, xml_parser_get_property_bool (node, "REPEAT-TRACK", 0));
       }
 
       else if (!strcasecmp (node->name, "ENTRY"))
@@ -471,7 +497,7 @@ static int playlist_load (const char *fname)
       {
 	/* Attributes: HREF, CLIENTBIND */
 	play_item_t *play_item =
-	  play_item_new (NULL, xml_parser_get_property (node, "HREF"), 0);
+	  play_item_new (NULL, xml_parser_get_property (node, "HREF"), 0, 0);
 	GtkTreeIter  iter;
 	gtk_list_store_append (pl_store, &iter);
 	gtk_list_store_set (pl_store, &iter,
@@ -525,11 +551,12 @@ void playlist_save (char *file)
 
     fprintf
       (f, "<ASX VERSION=\"3.0\">\n"
-          "  <SETTINGS REPEAT=\"%s\" RANDOM=\"%s\"/>\n",
+          "  <SETTINGS REPEAT='%s' REPEAT-TRACK='%s' RANDOM='%s'/>\n",
        tf[!!gtk_toggle_button_get_active (repeat_button)],
+       tf[!!gtk_toggle_button_get_active (repeat_track_button)],
        tf[!!gtk_toggle_button_get_active (random_button)]);
 
-    if (gtk_tree_model_get_iter_first (GTK_TREE_MODEL (pl_store), &iter))
+    if (gtk_tree_model_get_iter_first (pl_model, &iter))
     {
       do
       {
@@ -540,7 +567,7 @@ void playlist_save (char *file)
 	  fputs (xml, f);
 	  free (xml);
 	}
-      } while (gtk_tree_model_iter_next (GTK_TREE_MODEL(pl_store), &iter));
+      } while (gtk_tree_model_iter_next (pl_model, &iter));
     }
 
     fprintf (f, "</ASX>\n");
@@ -571,7 +598,7 @@ static void save_as_cb (GtkWidget* widget, gpointer data)
 
 static int playlist_clip (int pos)
 {
-  int items = gtk_tree_model_iter_n_children (GTK_TREE_MODEL (pl_store), NULL);
+  int items = gtk_tree_model_iter_n_children (pl_model, NULL);
   return (pos == -1) ? (items - (items != 0))
 		     : (pos < 0 || pos >= items) ? 0 : pos;
 }
@@ -584,9 +611,9 @@ play_item_t *playlist_get_item (int pos)
        && gtk_tree_selection_get_selected (sel, NULL, &iter))
       ||
       ((pos = playlist_clip (pos)) >= 0 /* nth item */
-       && gtk_tree_model_iter_nth_child
-	    (GTK_TREE_MODEL (pl_store), &iter, NULL, pos)))
+       && gtk_tree_model_iter_nth_child (pl_model, &iter, NULL, pos)))
     return peek_play_item (&iter);
+    
   else
     return NULL;
 }
@@ -607,7 +634,7 @@ play_item_t *playlist_get_current_item (void)
 
 int playlist_size (void)
 {
-  return gtk_tree_model_iter_n_children (GTK_TREE_MODEL (pl_store), NULL);
+  return gtk_tree_model_iter_n_children (pl_model, NULL);
 }
 
 int playlist_add (play_item_t *play_item, gint ins_pos)
@@ -640,7 +667,7 @@ int playlist_add (play_item_t *play_item, gint ins_pos)
    * find out entry number in list
    */
 
-  path = gtk_tree_model_get_path (GTK_TREE_MODEL (pl_store), &iter);
+  path = gtk_tree_model_get_path (pl_model, &iter);
   indices = gtk_tree_path_get_indices (path);
   ret = indices[0];
   gtk_tree_path_free (path);
@@ -677,7 +704,7 @@ static gint playlist_add_int (const char *mrl, gint ins_pos)
       if (!strcmp (playlist_get_item (i)->mrl, mrl))
         return i;
 
-  play_item = play_item_new (NULL, mrl, 0);
+  play_item = play_item_new (NULL, mrl, 0, 0);
   play_item_check_autoplay (play_item);
 
   return playlist_add (play_item, ins_pos);
@@ -934,7 +961,7 @@ pl_smil_fetch (pl_xml_info_t *info, void *data)
 
       char *mrl = xml_parser_get_property (info->node, "SRC");
       if (mrl)
-	return play_item_new (NULL, mrl, 0);
+	return play_item_new (NULL, mrl, 0, 0);
     }
   }
 }
@@ -987,10 +1014,18 @@ pl_ram_fetch (const char *const *lines, size_t index, void *data)
     return (play_item_t *) -1;
 
   /* it's probably an RTSP or PNM MRL, but we also match HTTP here */
-  if (!strncasecmp (lines[index], "rtsp://", 7)
-      || !strncasecmp (lines[index], "pnm://", 6)
-      || !strncasecmp (lines[index], "http://", 7))
-    return play_item_new (NULL, lines[index], 0);
+  if (!strncasecmp (lines[index], "rtsp://", 7) ||
+      !strncasecmp (lines[index], "pnm://", 6))
+  {
+    char line[strlen (lines[index]) + 1];
+    strcpy (line, lines[index]);
+    char *q = strchr (line, '?');
+    if (q)
+      *q = 0; /* strip off '?...' */
+    return play_item_new (NULL, line, 0, 0);
+  }
+  if (!strncasecmp (lines[index], "http://", 7))
+    return play_item_new (NULL, lines[index], 0, 0);
 
   return NULL;
 }
@@ -1021,7 +1056,7 @@ pl_m3u_fetch (const char *const *lines, size_t index, void *data)
 		       ? "#demux:ogg"
 		       : NULL;
   char *tmp = g_strconcat (lines[index], suffix, NULL);
-  play_item_t *item = play_item_new (NULL, tmp, 0);
+  play_item_t *item = play_item_new (NULL, tmp, 0, 0);
   free (tmp);
 
   return item;
@@ -1101,7 +1136,7 @@ pl_pls_fetch (const char *const *lines, size_t index, void *data)
 	title[pos] = 0;
     }
 
-    play_item_t *ret = play_item_new (title, file, 0);
+    play_item_t *ret = play_item_new (title, file, 0, 0);
     g_free (file);
 
     return ret;
@@ -1208,7 +1243,7 @@ static int playlist_load_asx (const char *mrl, gint ins_pos)
 		logprintf ("playlist: HREF property detected\n"
 			   "playlist: found an mrl: %s\n", property->value);
 
-		item = play_item_new (title, property->value, 0);
+		item = play_item_new (title, property->value, 0, 0);
 
 		if (ret >= 0)
 		  playlist_add (item, ins_pos);
@@ -1323,8 +1358,8 @@ static int playlist_add_playlist_mrl (const char *mrl, gint ins_pos)
     if (!strncasecmp (extension, ".m3u", 4))
       return playlist_load_m3u (mrl, ins_pos);
 
-    if (!strcasecmp (extension, ".ra") ||
-	!strcasecmp (extension, ".rm") ||
+    if (/*!strcasecmp (extension, ".ra") ||*/
+	/*!strcasecmp (extension, ".rm") ||*/
 	!strncasecmp (extension, ".ram", 4) ||
 	!strncasecmp (extension, ".rpm", 4) ||
 	!strncasecmp (extension, ".lsc", 4) ||
@@ -1460,8 +1495,7 @@ static void set_mark_play_item (gboolean state)
   if (cur_list_pos >= 0 && cur_list_pos < playlist_size ())
   {
     GtkTreeIter iter;
-    gtk_tree_model_iter_nth_child (GTK_TREE_MODEL (pl_store), &iter, NULL,
-				   cur_list_pos);
+    gtk_tree_model_iter_nth_child (pl_model, &iter, NULL, cur_list_pos);
     gtk_list_store_set (pl_store, &iter, COLUMN_MARKING, state ? "•" : "", -1);
   }
 }
@@ -1471,7 +1505,7 @@ void playlist_logo (gpointer wait_if_non_null)
   unmark_play_item ();
   cur_item_dispose ();
   logo_mode = 1;
-  player_launch (NULL, logo_mrl, 0, 0);
+  player_launch (NULL, logo_mrl, 0, 0, 0);
   if (wait_if_non_null)
     player_wait ();
 }
@@ -1507,6 +1541,18 @@ static gboolean row_activated_lcb (GtkWidget *widget, GtkTreePath *path,
   return FALSE;
 }
 
+static gboolean row_clicked_lcb (GtkWidget *widget, GdkEventButton *ev,
+				 gpointer data)
+{
+  return list_menu_click_cb (tree_view, ev, (GtkMenu **) data);
+}
+
+static gboolean key_press_lcb (GtkWidget *widget, GdkEventButton *ev,
+				 gpointer data)
+{
+  return list_menu_key_press_cb (tree_view, ev, (GtkMenu **) data);
+}
+
 static int has_been_played (int pos)
 {
   return playlist_get_item (pos)->played;
@@ -1516,6 +1562,12 @@ static void play_next (void)
 {
   int is_repeat, is_random;
 
+  if (gtk_toggle_button_get_active (repeat_track_button))
+  {
+    playlist_play (cur_list_pos);
+    return;
+  }
+
   is_repeat = gtk_toggle_button_get_active (repeat_button);
   is_random = gtk_toggle_button_get_active (random_button);
 
@@ -1570,6 +1622,97 @@ static void play_next (void)
   }
 }
 
+#ifdef XINE_MSG_AUTHENTICATION_NEEDED
+static gboolean auth_validate (const char *text)
+{
+  if (!text || !*text)
+    return FALSE;
+  while (*text)
+  {
+    unsigned char c = (unsigned char) *text++;
+    if (c < 33 || c == '/' || c == ':' || c == '@')
+      return FALSE;
+  }
+  return TRUE;
+}
+
+static int display_auth (int args, const char *mrl)
+{
+  if (GTK_WIDGET_VISIBLE ((GtkWidget *) auth.dbox))
+    return 0;
+
+  const char *host = strstr (mrl, "://");
+  if (!host)
+    return 0;
+
+  int retval = 2;
+
+  char *txt = g_strdup_printf (_("Authorisation required for %s"), mrl);
+  gtk_window_set_title (GTK_WINDOW (auth.dbox), txt);
+  free (txt);
+  gtk_label_set_text (auth.label, args > 1 ? mrl + strlen (mrl) + 1 : "");
+  gtk_entry_set_text (auth.name, "");
+  gtk_entry_set_text (auth.pass, "");
+  gtk_widget_grab_focus ((GtkWidget *) auth.name);
+
+  if (gtk_dialog_run (auth.dbox) == GTK_RESPONSE_OK)
+  {
+    const char *name = gtk_entry_get_text (auth.name);
+    const char *pass = gtk_entry_get_text (auth.pass);
+    if (!auth_validate (name) || !auth_validate (pass))
+    {
+      display_error (FROM_GXINE, "%s", _("Authentication data contains invalid characters."));
+      goto ret;
+    }
+
+    host += 3;
+    const char *at = strchr (host, '@');
+    const char *slash = strchr (host, ':');
+    if (at && slash && at > slash)
+      at = NULL;
+    if (at)
+    {
+      const char *colon = strchr (host, ':');
+      if (colon >= at)
+        at = NULL;
+    }
+
+    if (at)
+      txt = g_strdup_printf ("%.*s%s:%s%s", (int)(host - mrl), mrl, name, pass, at);
+    else
+      txt = g_strdup_printf ("%.*s%s:%s@%s", (int)(host - mrl), mrl, name, pass, host);
+
+    play_item_t *play_item;
+    int pos;
+    for (pos = 0; pos < playlist_size (); ++pos)
+      if (!strcmp ((play_item = playlist_get_item (pos))->mrl, mrl))
+	break;
+    if (pos == playlist_size ())
+    {
+      pos = playlist_add (play_item_new (NULL, txt, 0, 0), -1);
+      free (txt);
+    }
+    else
+    {
+      free (play_item->mrl);
+      play_item->mrl = txt;
+      GtkTreeIter iter;
+      gtk_tree_model_iter_nth_child (pl_model, &iter, NULL, pos);
+      gtk_list_store_set (pl_store, &iter, COLUMN_MRL, play_item->mrl, -1);
+    }
+    playlist_play (pos);
+    retval = 1;
+  }
+
+ret:
+  gtk_widget_hide ((GtkWidget *) auth.dbox);
+  gtk_entry_set_text (auth.name, "");
+  gtk_entry_set_text (auth.pass, "");
+
+  return retval;
+}
+#endif /* XINE_MSG_AUTHENTICATION_NEEDED */
+
 static void xine_event_cb (void *user_data, const xine_event_t *event)
 {
   /*
@@ -1587,7 +1730,13 @@ static void xine_event_cb (void *user_data, const xine_event_t *event)
       gdk_threads_leave();
       se_eval (gse, "eval (event.stream_end);", NULL, NULL, NULL, "event.stream_end");
       gdk_threads_enter();
-      play_next ();
+#ifdef XINE_PARAM_GAPLESS_SWITCH
+      xine_set_param (stream, XINE_PARAM_GAPLESS_SWITCH, 1);
+#endif
+      if (gtk_toggle_button_get_active (repeat_track_button))
+	playlist_play (cur_list_pos);
+      else
+	play_next ();
     }
     gdk_threads_leave();
     break;
@@ -1623,9 +1772,11 @@ static void xine_event_cb (void *user_data, const xine_event_t *event)
 
   case XINE_EVENT_UI_SET_TITLE:
   case XINE_EVENT_FRAME_FORMAT_CHANGE: /* in case of DVB etc. */
-    gdk_threads_enter();
+  case XINE_EVENT_UI_CHANNELS_CHANGED:
+    gdk_threads_enter ();
     infobar_show_metadata (infobars);
     playlist_check_set_title ();
+    update_subtitles_menu ();
     gdk_threads_leave();
     break;
 
@@ -1650,7 +1801,7 @@ typedef struct {
       if (ref->alternative == 0)
       {
 	play_item_t *item = play_item_new (*title ? title : NULL, ref->mrl,
-					   ref->start_time);
+					   ref->start_time, ref->duration);
 	item->type = PLAY_ITEM_PLAYLIST;
 	gdk_threads_enter ();
 
@@ -1662,7 +1813,7 @@ typedef struct {
 	  /* remove the item which triggered this reference event */
 	  /*
 	  GtkTreeIter iter;
-	  gtk_tree_model_iter_nth_child (GTK_TREE_MODEL (pl_store), &iter,
+	  gtk_tree_model_iter_nth_child (pl_model, &iter,
 					 NULL, cur_list_pos--);
 	  gtk_list_store_remove (pl_store, &iter);
 	  */
@@ -1683,7 +1834,7 @@ typedef struct {
 
       if (ref->alternative == 0)
       {
-	play_item_t *item = play_item_new (NULL, ref->mrl, 0);
+	play_item_t *item = play_item_new (NULL, ref->mrl, 0, 0);
 	item->type = PLAY_ITEM_PLAYLIST;
 	gdk_threads_enter ();
 
@@ -1695,7 +1846,7 @@ typedef struct {
 	  /* remove the item which triggered this reference event */
 	  /*
 	  GtkTreeIter iter;
-	  gtk_tree_model_iter_nth_child (GTK_TREE_MODEL (pl_store), &iter,
+	  gtk_tree_model_iter_nth_child (pl_model, &iter,
 					 NULL, cur_list_pos--);
 	  gtk_list_store_remove (pl_store, &iter);
 	  */
@@ -1719,7 +1870,7 @@ typedef struct {
       switch (data->type)
       {
       default:
-        g_print (_("xine_event_cb: unknown XINE_MSG_*"));
+        g_print (_("xine_event_cb: unknown XINE_MSG_*\n"));
       case XINE_MSG_NO_ERROR:
       case XINE_MSG_AUDIO_OUT_UNAVAILABLE:
       case XINE_MSG_ENCRYPTED_SOURCE:
@@ -1744,10 +1895,32 @@ typedef struct {
       case XINE_MSG_UNKNOWN_DEVICE:
         msg = display_error;
         break;
+
+#ifdef XINE_MSG_AUTHENTICATION_NEEDED
+      /* new in xine-lib 1.2 */
+      case XINE_MSG_AUTHENTICATION_NEEDED:
+	msg = display_error;
+	if (data->num_parameters >= 1)
+	{
+	  play_exec_error_suppress_next ();
+	  switch (display_auth (data->num_parameters,
+				(const char *)data + data->parameters))
+	  {
+	  case 2:
+	    play_next ();
+	  case 1:
+	    msg = NULL;
+	    break;
+	  }
+	}
+	break;
+#endif
       }
 
       /* TODO: provide customized messages, hints... */
-      if (data->num_parameters)
+      if (!msg)
+	/**/;
+      else if (data->num_parameters)
       {
         int i;
 	const char *param = (char *)data + data->parameters;
@@ -1794,8 +1967,7 @@ void playlist_check_set_title (void)
   {
     GtkTreeIter iter;
 
-    gtk_tree_model_iter_nth_child (GTK_TREE_MODEL (pl_store), &iter,
-				   NULL, cur_list_pos);
+    gtk_tree_model_iter_nth_child (pl_model, &iter, NULL, cur_list_pos);
     play_item_t *play_item = peek_play_item (&iter);
 
     if (play_item->untitled)
@@ -1961,7 +2133,7 @@ static JSBool js_playlist_add (JSContext *cx, JSObject *obj, uintN argc,
 		  ? JS_GetStringBytes (JS_ValueToString (cx, argv[1]))
 		  : NULL;
     logprintf ("playlist_add: MRL=%s title=%s\n", mrl, title);
-    play_item_t *play_item = play_item_new (title, mrl, 0);
+    play_item_t *play_item = play_item_new (title, mrl, 0, 0);
     item = playlist_add (play_item, -1);
   }
   else
@@ -1986,13 +2158,13 @@ static JSBool js_playlist_delete (JSContext *cx, JSObject *obj, uintN argc,
   int32 pos;
   JS_ValueToInt32 (cx, argv[0], &pos);
 
-  int count = gtk_tree_model_iter_n_children (GTK_TREE_MODEL (pl_store), NULL);
+  int count = gtk_tree_model_iter_n_children (pl_model, NULL);
   if (pos < 0)
     pos = count - pos;
   if (pos >= 0 && pos < count)
   {
     GtkTreeIter iter;
-    gtk_tree_model_iter_nth_child (GTK_TREE_MODEL (pl_store), &iter, NULL, pos);
+    gtk_tree_model_iter_nth_child (pl_model, &iter, NULL, pos);
 
     if (pos == cur_list_pos && !logo_mode)
       play_next ();
@@ -2040,7 +2212,7 @@ static JSBool js_mrl_browser_refresh (JSContext *cx, JSObject *obj, uintN argc,
   se_log_fncall_checkinit ("mrl_browser_refresh");
 
   if (cur_list_pos < 0
-      || !gtk_tree_model_iter_nth_child (GTK_TREE_MODEL (pl_store), &iter, NULL,
+      || !gtk_tree_model_iter_nth_child (pl_model, &iter, NULL,
 					 cur_list_pos)
       || !item_marked_current (&iter))
   {
@@ -2079,7 +2251,8 @@ void playlist_play_from (int list_pos, int pos, int pos_time)
     return;
   }
 
-  play_item_play (play_item, pos, pos_time);
+  play_item_play (play_item, pos, pos_time,
+		  play_item->duration + play_item->start_time - pos_time);
 
   if (browse_cb)
     g_source_remove (browse_cb);
@@ -2100,13 +2273,14 @@ static const GtkActionEntry buttons_data[] = {
   { "open",	GTK_STOCK_OPEN,		NULL,			NULL,		N_("Open a playlist file"),			open_cb },
   { "saveas",	GTK_STOCK_SAVE_AS,	NULL,			NULL,		N_("Save the playlist file"),			save_as_cb },
   { "clear",	GTK_STOCK_CLEAR,	NULL,			NULL,		N_("Clear the playlist"),			clear_cb },
-  { "fromfile",	GTK_STOCK_OPEN,		NULL,			NULL,		N_("Add files to the playlist"),		add_cb },
-  { "new",	GTK_STOCK_NEW,		NULL,			NULL,		NULL /* FIXME: post-0.5.5 */,			new_cb },
+  { "fromfile",	GTK_STOCK_OPEN,		N_("_Add files"),	NULL,		N_("Add files to the playlist"),		add_cb },
+  { "new",	GTK_STOCK_NEW,		NULL,			NULL,		N_("New playlist item"),			new_cb },
   { "edit",	GTK_STOCK_EDIT,		NULL,			"<Control>E",	N_("Edit the selected item"),			edit_cb },
   { "delete",	GTK_STOCK_DELETE,	NULL,			"Delete",	N_("Delete the selected item"),			del_cb },
   { "copy",	GTK_STOCK_COPY,		NULL,			NULL,		NULL,						copy_cb },
   { "cut",	GTK_STOCK_CUT,		NULL,			NULL,		NULL,						cut_cb },
   { "paste",	GTK_STOCK_PASTE,	NULL,			NULL,		NULL,						paste_cb },
+  { "play",	GTK_STOCK_MEDIA_PLAY,	NULL,			NULL,		NULL,						play_cb },
   { "mm",	GXINE_MEDIA_MARK,	N_("Add to _media marks"), "<Control>M",N_("Add the selected item to the media marks list"), make_mediamark_cb },
   { "prev",	GTK_STOCK_MEDIA_PREVIOUS, NULL,			NULL,		N_("Play previous item"),			playlist_prev_cb },
   { "next",	GTK_STOCK_MEDIA_NEXT,	NULL,			NULL,		N_("Play next item"),				playlist_next_cb },
@@ -2128,10 +2302,36 @@ static const char buttons_structure[] =
       "<toolitem action='paste' />\n"
       "<toolitem action='mm' />\n"
       "<separator />\n"
+      "<toolitem action='play' />\n"
       "<toolitem action='prev' />\n"
       "<toolitem action='next' />\n"
     "</toolbar>\n"
-  "</ui>";
+    "<popup name='item_menu'>\n"
+      "<menuitem action='play' />\n"
+      "<menuitem action='mm' />\n"
+      "<separator />\n"
+      "<menuitem action='edit' />\n"
+      "<menuitem action='delete' />\n"
+      "<menuitem action='copy' />\n"
+      "<menuitem action='cut' />\n"
+      "<menuitem action='paste' />\n"
+      "<separator />\n"
+      "<menuitem action='prev' />\n"
+      "<menuitem action='next' />\n"
+    "</popup>\n"
+    "<popup name='list_menu'>\n"
+      "<menuitem action='open' />\n"
+      "<menuitem action='saveas' />\n"
+      "<menuitem action='clear' />\n"
+      "<separator />\n"
+      "<menuitem action='fromfile' />\n"
+      "<menuitem action='new' />\n"
+      "<menuitem action='paste' />\n"
+      "<separator />\n"
+      "<menuitem action='prev' />\n"
+      "<menuitem action='next' />\n"
+    "</popup>\n"
+  "</ui>"; 
 
 static void sel_changed_cb (GtkTreeSelection *sel, gpointer data)
 {
@@ -2139,7 +2339,7 @@ static void sel_changed_cb (GtkTreeSelection *sel, gpointer data)
     "edit", NULL
   };
   static const char *const move_items[] = {
-    "delete", "mm", "copy", "cut", NULL
+    "play", "delete", "mm", "copy", "cut", NULL
   };
   GtkTreeIter iter;
   if (gtk_tree_selection_get_selected (sel, NULL, &iter))
@@ -2157,34 +2357,17 @@ static void sel_changed_cb (GtkTreeSelection *sel, gpointer data)
 static void check_list_empty (void)
 {
   static const char *const items[] = {
-    "saveas", "clear", "prev", "next", NULL
+    "saveas", "clear", NULL
   };
-  ui_mark_active (pl_ui, items,
-		  gtk_tree_model_iter_n_children (GTK_TREE_MODEL (pl_store),
-						  NULL));
+  static const char *const one_items[] = {
+    "prev", "next", NULL
+  };
+  unsigned int count = gtk_tree_model_iter_n_children (pl_model, NULL);
+  ui_mark_active (pl_ui, items, count != 0);
+  ui_mark_active (pl_ui, one_items, count > 1);
   ui_set_status (UI_CURRENT_STATE);
 }
 
-static void get_playlist_repeat (se_prop_t *prop, se_prop_read_t *value)
-{
-  value->i = gtk_toggle_button_get_active (repeat_button);
-}
-
-static void get_playlist_random (se_prop_t *prop, se_prop_read_t *value)
-{
-  value->i = gtk_toggle_button_get_active (random_button);
-}
-
-static void set_playlist_repeat (int v)
-{
-  gtk_toggle_button_set_active (repeat_button, v);
-}
-
-static void set_playlist_random (int v)
-{
-  gtk_toggle_button_set_active (random_button, v);
-}
-
 void playlist_init (void)
 {
   GtkWidget *hbox, *scrolled_window;
@@ -2198,6 +2381,26 @@ void playlist_init (void)
 
   srand (time (NULL));
 
+  /*
+   * buttons & menus
+   */
+
+  pl_ui = ui_create_manager ("file", dlg);
+  GtkActionGroup *actions = ui_get_action_group (pl_ui);
+  gtk_action_group_add_actions (actions, buttons_data,
+				G_N_ELEMENTS (buttons_data), (GtkWidget *)tree_view);
+  g_object_set (gtk_action_group_get_action (actions, "mm"),
+		"short-label", N_("_Media mark"), NULL);
+  gtk_ui_manager_add_ui_from_string (pl_ui, buttons_structure, -1, &error);
+  if (error)
+  {
+    g_printerr (_("playlist XML: %s\n"), error->message);
+    g_clear_error (&error);
+  }
+  menu[0] = gtk_ui_manager_get_widget (pl_ui, "/ui/list_menu");
+  menu[1] = gtk_ui_manager_get_widget (pl_ui, "/ui/item_menu");
+  gtk_action_group_connect_accelerators (actions);
+
   /*
    * window
    */
@@ -2205,51 +2408,43 @@ void playlist_init (void)
   dlg = gtk_dialog_new_with_buttons (_("Edit playlist..."), NULL, 0,
 				     GTK_STOCK_CLOSE, GTK_RESPONSE_CLOSE,
 				     NULL);
-  gtk_window_set_default_size (GTK_WINDOW (dlg), 500, 400);
+  gtk_window_set_default_size (GTK_WINDOW (dlg), 520, 400);
   g_object_connect (G_OBJECT (dlg),
+	"signal::map-event", G_CALLBACK (map_cb), NULL,
+	"signal::button-press-event", G_CALLBACK(list_menu_popup_cb), menu[0],
+	"signal::key-press-event", G_CALLBACK(key_press_lcb), menu,
 	"signal::delete-event", G_CALLBACK (close_cb), NULL,
 	"signal::response", G_CALLBACK(close_cb), NULL,
 	NULL);
 
   vbox = GTK_BOX (GTK_DIALOG (dlg)->vbox);
 
+  gtk_box_pack_start (vbox, gtk_ui_manager_get_widget (pl_ui, "/ui/toolbar"),
+		      FALSE, FALSE, 0);
+
   /*
    * init tree store & view
    */
 
   pl_store = gtk_list_store_new (5, G_TYPE_STRING, G_TYPE_STRING,
 				 G_TYPE_POINTER, G_TYPE_STRING, G_TYPE_STRING);
+  pl_model = GTK_TREE_MODEL (pl_store);
   g_object_connect (G_OBJECT (pl_store),
 	"signal::row-deleted", G_CALLBACK (check_list_empty), NULL,
 	"signal::row-inserted", G_CALLBACK (check_list_empty), NULL,
 	NULL);
 
-  tree_view = GTK_TREE_VIEW (gtk_tree_view_new_with_model (GTK_TREE_MODEL (pl_store)));
+  tree_view = GTK_TREE_VIEW (gtk_tree_view_new_with_model (pl_model));
   gtk_tree_view_set_rules_hint (tree_view, TRUE);
-  g_signal_connect (G_OBJECT(tree_view), "row-activated",
-		    G_CALLBACK(row_activated_lcb), NULL);
+  g_object_connect (G_OBJECT(tree_view),
+	"signal::row-activated", G_CALLBACK(row_activated_lcb), NULL,
+	"signal::button-press-event", G_CALLBACK(row_clicked_lcb), menu,
+	NULL);
 
   sel = gtk_tree_view_get_selection (tree_view);
   g_signal_connect (G_OBJECT (sel), "changed",
 		    G_CALLBACK (sel_changed_cb), NULL);
 
-  /*
-   * buttons
-   */
-
-  pl_ui = ui_create_manager ("file", dlg);
-  gtk_action_group_add_actions (ui_get_action_group (pl_ui), buttons_data,
-				G_N_ELEMENTS (buttons_data), (GtkWidget *)tree_view);
-  gtk_ui_manager_add_ui_from_string (pl_ui, buttons_structure, -1, &error);
-  if (error)
-  {
-    g_printerr (_("playlist XML: %s\n"), error->message);
-    g_clear_error (&error);
-  }
-  gtk_box_pack_start (vbox, gtk_ui_manager_get_widget (pl_ui, "/toolbar"),
-		      FALSE, FALSE, 0);
-  gtk_action_group_connect_accelerators (ui_get_action_group (pl_ui));
-
   sel_changed_cb (sel, NULL);
 
   /*
@@ -2302,10 +2497,13 @@ void playlist_init (void)
   gtk_box_pack_start (vbox, hbox, FALSE, FALSE, 2);
 
   repeat_button = (GtkToggleButton *)gtk_check_button_new_with_mnemonic (_("_Repeat"));
-  gtk_box_pack_start (GTK_BOX (hbox), repeat_button, FALSE, FALSE, 2);
+  gtk_box_pack_start (GTK_BOX (hbox), (GtkWidget *)repeat_button, FALSE, FALSE, 2);
+
+  repeat_track_button = (GtkToggleButton *)gtk_check_button_new_with_mnemonic (_("Repeat _track"));
+  gtk_box_pack_start (GTK_BOX (hbox), (GtkWidget *)repeat_track_button, FALSE, FALSE, 2);
 
   random_button = (GtkToggleButton *)gtk_check_button_new_with_mnemonic (_("_Random"));
-  gtk_box_pack_start (GTK_BOX (hbox), random_button, FALSE, FALSE, 2);
+  gtk_box_pack_start (GTK_BOX (hbox), (GtkWidget *)random_button, FALSE, FALSE, 2);
 
   sprintf (logo_mrl, "%s/logo."LOGO_FORMAT, logodir);
   logo_mode = 1;
@@ -2313,6 +2511,36 @@ void playlist_init (void)
   playlist_load (NULL);
   check_list_empty ();
 
+  /*
+   * authorisation dbox
+   */
+
+  auth.dbox = GTK_DIALOG (gtk_dialog_new_with_buttons
+			    ("", (GtkWindow *) app,
+			     GTK_DIALOG_MODAL | GTK_DIALOG_NO_SEPARATOR,
+			     GTK_STOCK_CANCEL, GTK_RESPONSE_DELETE_EVENT,
+			     GTK_STOCK_OPEN, GTK_RESPONSE_OK,
+			     NULL));
+  gtk_dialog_set_default_response (auth.dbox, GTK_RESPONSE_OK);
+  auth.label = GTK_LABEL (gtk_label_new (""));
+  auth.name = GTK_ENTRY (gtk_entry_new_with_max_length (64));
+  auth.pass = GTK_ENTRY (gtk_entry_new_with_max_length (128));
+  gtk_entry_set_visibility (GTK_ENTRY (auth.pass), FALSE);
+  GtkWidget *table = gtk_table_new (2, 2, FALSE);
+  add_table_row_items (table, 0, 1, _("User name"), auth.name, NULL);
+  add_table_row_items (table, 1, 1, _("Password"), auth.pass, NULL);
+  vbox = GTK_BOX (gtk_hbox_new (0, 2));
+  gtk_box_pack_start (vbox,
+		      gtk_image_new_from_stock (GTK_STOCK_DIALOG_AUTHENTICATION,
+						GTK_ICON_SIZE_DIALOG),
+		      FALSE, FALSE, 0);
+  gtk_box_pack_start (vbox, table, TRUE, TRUE, 0);
+  gtk_box_pack_start ((GtkBox *) auth.dbox->vbox, (GtkWidget *) auth.label,
+		      TRUE, TRUE, 2);
+  gtk_box_pack_start ((GtkBox *) auth.dbox->vbox, (GtkWidget *) vbox,
+		      TRUE, TRUE, 0);
+  gtk_widget_show_all ((GtkWidget *) auth.dbox->vbox);
+
   /*
    * event handling
    */
@@ -2342,8 +2570,9 @@ void playlist_init (void)
       { NULL }
     };
     static const ui_property_t props[] = {
-      { "repeat", N_("v=bool, toggle()"), 0, get_playlist_repeat, set_playlist_repeat },
-      { "random", N_("v=bool, toggle()"), 0, get_playlist_random, set_playlist_random },
+      { "repeat", N_("v=bool, toggle()"), UI_GTK_WIDGET (repeat_button, gtk_toggle_button_get_active, gtk_toggle_button_set_active) },
+      { "repeat_track", N_("v=bool, toggle()"), UI_GTK_WIDGET (repeat_track_button, gtk_toggle_button_get_active, gtk_toggle_button_set_active) },
+      { "random", N_("v=bool, toggle()"), UI_GTK_WIDGET (random_button, gtk_toggle_button_get_active, gtk_toggle_button_set_active) },
       { NULL }
     };
     se_o_t *playlist = se_create_object (gse, NULL, "playlist", NULL,
