@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2002-2005 the xine-project
+ * Copyright (C) 2002-2007 the xine-project
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -16,8 +16,6 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
  * USA
  *
- * $Id: settings.c,v 1.36 2006/03/27 22:11:25 dsalt Exp $
- *
  * settings implementation
  */
 
@@ -53,6 +51,7 @@ struct slider_tab_s {
 
 static const struct slider_s settings_video[] = {
   { N_("A/V sync"),	"gxine-settings-av-sync",	Control_AV_SYNC },
+  { N_("SPU sync"),	"gxine-settings-spu",		Control_SPU_SYNC },
   { N_("Hue"),		"gxine-settings-hue",		Control_HUE },
   { N_("Saturation"),	"gxine-settings-saturation",	Control_SATURATION },
   { N_("Contrast"),	"gxine-settings-contrast",	Control_CONTRAST },
@@ -97,7 +96,7 @@ struct slider_window_s {
 };
 
 static struct slider_window_s settings_window = {
-  N_("Settings"), settings_page, NULL, FALSE
+  N_("Controls"), settings_page, NULL, FALSE
 };
 
 static const struct slider_s *
@@ -194,13 +193,17 @@ JSFUNC (settings, clear)
 struct slider_data_s {
   GtkLabel *label;
   gdouble value;
+  uintptr_t sequence;
 };
 
 static gboolean slider_set_text (struct slider_data_s *data)
 {
-  char n[12];
-  sprintf (n, "%.0f", data->value);
-  gtk_label_set_text (data->label, n);
+  if (data->sequence >= (intptr_t) g_object_get_data (data->label, "slider_seq"))
+  {
+    char n[12];
+    sprintf (n, "%.0f", data->value);
+    gtk_label_set_text (data->label, n);
+  }
   free (data);
   return FALSE;
 }
@@ -209,8 +212,10 @@ static gchar *slider_format_value (GtkScale *scale, gdouble value,
 				   gpointer data)
 {
   struct slider_data_s *slider = malloc (sizeof (struct slider_data_s));
+  slider->sequence = (intptr_t) g_object_get_data (data, "slider_seq");
+  g_object_set_data (data, "slider_seq", (gpointer) (++slider->sequence));
   slider->label = data;
-  slider->value = value;
+  slider->value = scale->range.adjustment->value; /* don't trust the input value */
   g_idle_add ((GSourceFunc) slider_set_text, slider);
   return g_strdup ("");
 }
@@ -225,15 +230,16 @@ static void sliders_create (struct slider_window_s *window)
 
   window->window = gtk_dialog_new_with_buttons (gettext (window->title),
 						NULL, 0, NULL);
-  gtk_dialog_add_action_widget (GTK_DIALOG(window->window),
+  GtkDialog *dbox = GTK_DIALOG (window->window);
+  gtk_dialog_add_action_widget (dbox,
 				ui_button_new_stock_mnemonic
 				  (GTK_STOCK_CLEAR, _("_Default")),
 				1);
-  gtk_dialog_add_buttons (GTK_DIALOG(window->window),
+  gtk_dialog_add_buttons (dbox,
 			GTK_STOCK_UNDO, GTK_RESPONSE_REJECT,
 			GTK_STOCK_CLOSE, GTK_RESPONSE_DELETE_EVENT,
 			NULL);
-  gtk_window_set_default_size (GTK_WINDOW (window->window), 500, 150);
+  gtk_window_set_default_size (GTK_WINDOW (dbox), 500, 150);
   hide_on_delete (window->window, &window->is_visible);
   ui_add_undo_response (window->window, NULL);
   g_signal_connect (G_OBJECT(window->window), "response",
@@ -255,31 +261,24 @@ static void sliders_create (struct slider_window_s *window)
       GtkAdjustment *adj;
       GtkWidget *w, *v;
 
-      w = gtk_label_new (gettext (sliders[i].label));
-      gtk_misc_set_alignment (GTK_MISC (w), 1, 0.5);
-      gtk_table_attach (GTK_TABLE(table), w,
+      gtk_table_attach (GTK_TABLE(table),
+			ui_label_new_with_xalign (gettext (sliders[i].label), 1),
 			0, 1, i, i+1, GTK_FILL, GTK_FILL, 2, 2);
 
-      v = gtk_label_new ("");
-      gtk_misc_set_alignment (GTK_MISC (v), 1, 0.5);
-      gtk_table_attach (GTK_TABLE(table), v,
-			1, 2, i, i+1, GTK_FILL, GTK_FILL, 2, 2);
+      v = ui_label_new_with_xalign ("", 1);
+      gtk_table_attach (GTK_TABLE(table), v, 1, 2, i, i+1, GTK_FILL, GTK_FILL, 2, 2);
 
       if (sliders[i].icon)
-      {
-	w = gtk_image_new_from_stock (sliders[i].icon, GTK_ICON_SIZE_MENU);
-	gtk_table_attach (GTK_TABLE(table), w,
+	gtk_table_attach (GTK_TABLE(table), gtk_image_new_from_stock (sliders[i].icon, GTK_ICON_SIZE_MENU),
 			  2, 3, i, i+1, GTK_FILL, GTK_FILL, 2, 2);
-      }
 
       adj = (GtkAdjustment *) ui_register_control_adjustment (sliders[i].type);
       if (adj->lower < min)
 	min = adj->lower;
       if (adj->upper > max)
 	max = adj->upper;
-      w = gtk_hscale_new (adj);
-      gtk_scale_set_value_pos (GTK_SCALE(w), GTK_POS_LEFT);
-      gtk_scale_set_digits (GTK_SCALE(w), 0);
+      w = ui_hscale_new ((GtkObject *)adj, GTK_POS_LEFT, 0);
+      ui_register_adjustment_widget (sliders[i].type, w);
       g_signal_connect (G_OBJECT(w), "format-value",
 			G_CALLBACK(slider_format_value), v);
       gtk_table_attach (GTK_TABLE(table), w,
@@ -292,7 +291,7 @@ static void sliders_create (struct slider_window_s *window)
       max = min;
 
     { /* create an 'invisible' label to set the width of the second column */
-      static const char pad[] = "____________";
+      static const char pad[] = "9999999999999";
       GtkWidget *w =
 	gtk_label_new (pad + sizeof (pad) - 1 -
 		       (max > (int) sizeof (pad) ? (int) sizeof (pad)
@@ -306,8 +305,7 @@ static void sliders_create (struct slider_window_s *window)
     }
   }
 
-  gtk_box_pack_start (GTK_BOX(GTK_DIALOG(window->window)->vbox),
-		      window->tabbox, TRUE, TRUE, 2);
+  gtk_box_pack_start (GTK_BOX(dbox->vbox), window->tabbox, TRUE, TRUE, 2);
 
   lock = xine_config_lookup_entry (xine, "audio.volume.remember_volume", &entry)
 	 && !entry.num_value;
