@@ -1,5 +1,5 @@
 /*
- * Copyright (C) 2004-2006 the xine-project
+ * Copyright (C) 2004-2007 the xine-project
  *
  * This program is free software; you can redistribute it and/or
  * modify it under the terms of the GNU General Public License as
@@ -16,8 +16,6 @@
  * Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301
  * USA
  *
- * $Id: ui.c,v 1.50 2006/04/08 21:34:50 dsalt Exp $
- *
  * various UI bits:
  * - button registry and update (state display etc.)
  * - slider/spinner adjustment creation and update
@@ -31,6 +29,8 @@
 #include <gtk/gtk.h>
 #include <gdk/gdkkeysyms.h>
 #include <pthread.h>
+/* for VO_CAP_HUE etc. (where available) */
+#include <xine/video_out.h>
 
 #include "ui.h"
 #include "utils.h"
@@ -45,6 +45,9 @@
  * UI button & adjustment handling
  */
 
+gboolean have_config = FALSE;
+int config_version = 0;
+
 gboolean no_recursion = FALSE;
 
 gboolean fs_toolbar_at_top = FALSE;
@@ -52,6 +55,8 @@ gboolean fs_toolbar_visible = FALSE;
 gboolean wm_toolbar_visible = TRUE;
 gboolean wm_toolbar_snap = TRUE;
 
+gboolean fs_always_sticky = FALSE;
+
 GtkIconSize icon_size_logo;
 
 static ui_status_t status = UI_CURRENT_STATE;
@@ -59,14 +64,16 @@ pthread_mutex_t status_lock = PTHREAD_MUTEX_INITIALIZER;
 
 static GSList *c_buttons[Control_Buttons] = { NULL };
 static GtkObject *c_adjustments[Control_Adjustments] = { NULL };
+static GSList *c_adj_widgets[Control_Adjustments] = { NULL };
 
 /* (Internal) Reflect play/pause/stop status in control buttons */
 
-static void ui_set_list (int index, const char *property, gboolean state)
+static void ui_set_list (GSList **list, int index, const char *property, gboolean state)
 {
   GSList *btn;
-  foreach_glist (btn, c_buttons[index])
-    g_object_set (G_OBJECT (btn->data), property, state, NULL);
+  foreach_glist (btn, list[index])
+    if (g_object_class_find_property (G_OBJECT_GET_CLASS (btn->data), property))
+      g_object_set (G_OBJECT (btn->data), property, state, NULL);
 }
 
 static void ui_int_set_status (ui_status_t status)
@@ -89,11 +96,11 @@ static void ui_int_set_status (ui_status_t status)
   no_recursion = TRUE;
   for (i = 0; i < Control_PlayerButtons; ++i)
   {
-    ui_set_list (i, "active", states[i]);
+    ui_set_list (c_buttons, i, "active", states[i]);
     if (i == Control_PLAY)
-      ui_set_list (i, "sensitive", !!playlist_size ());
+      ui_set_list (c_buttons, i, "sensitive", !!playlist_size ());
     else if (i == Control_PAUSE || i == Control_REWIND || i == Control_FASTFWD)
-      ui_set_list (i, "sensitive", !live);
+      ui_set_list (c_buttons, i, "sensitive", !live);
   }
   no_recursion = FALSE;
 }
@@ -156,8 +163,18 @@ typedef struct {
   gdouble start, min, max, step, page, pagesize;
   const char *setting, *jspref;
   int param;
+#ifdef VO_CAP_HUE
+  enum { CAP_NONE, CAP_AUDIO, CAP_VIDEO } type;
+  int cap;
+#endif
 } ui_adjustable_t;
 
+#ifdef VO_CAP_HUE
+# define XINE_PARAM_VO(X) XINE_PARAM_VO_##X, CAP_VIDEO, VO_CAP_##X
+#else
+# define XINE_PARAM_VO(X) XINE_PARAM_VO_##X
+#endif
+
 static const ui_adjustable_t ranges[] = {
   [Control_SEEKER]		= {     0,       0,  65535,    1,    10, 0, NULL },
   [Control_AUDIO_CHANNEL]	= {    -1,      -1,     32,    1,     1, 0, NULL,			 "ao_channel",	  XINE_PARAM_AUDIO_CHANNEL_LOGICAL },
@@ -165,10 +182,11 @@ static const ui_adjustable_t ranges[] = {
   [Control_COMPRESSOR]		= {   100,     100,   1000,   10,   100, 0, "gui.ao_compressor",	 "ao_compressor", XINE_PARAM_AUDIO_COMPR_LEVEL },
   [Control_AMPLIFIER]		= {   100,       0,    200,    1,    10, 0, "gui.ao_amplifier",		 "ao_amplifier",  XINE_PARAM_AUDIO_AMP_LEVEL },
   [Control_AV_SYNC]		= {     0, -180000, 180000, 1000, 10000, 0, "gui.av_sync",		 "av_sync",	  XINE_PARAM_AV_OFFSET },
-  [Control_HUE]			= { 32768,       0,  65535,  100,  1000, 0, "gui.vo_hue",		 "vo_hue",	  XINE_PARAM_VO_HUE },
-  [Control_SATURATION]		= { 32768,       0,  65535,  100,  1000, 0, "gui.vo_saturation",	 "vo_saturation", XINE_PARAM_VO_SATURATION },
-  [Control_CONTRAST]		= { 32768,       0,  65535,  100,  1000, 0, "gui.vo_contrast",		 "vo_contrast",   XINE_PARAM_VO_CONTRAST },
-  [Control_BRIGHTNESS]		= { 32768,       0,  65535,  100,  1000, 0, "gui.vo_brightness",	 "vo_brightness", XINE_PARAM_VO_BRIGHTNESS },
+  [Control_SPU_SYNC]		= {     0, -180000, 180000, 1000, 10000, 0, "gui.spu_sync",		 "spu_sync",	  XINE_PARAM_SPU_OFFSET },
+  [Control_HUE]			= { 32768,       0,  65535,  100,  1000, 0, "gui.vo_hue",		 "vo_hue",	  XINE_PARAM_VO(HUE) },
+  [Control_SATURATION]		= { 32768,       0,  65535,  100,  1000, 0, "gui.vo_saturation",	 "vo_saturation", XINE_PARAM_VO(SATURATION) },
+  [Control_CONTRAST]		= { 32768,       0,  65535,  100,  1000, 0, "gui.vo_contrast",		 "vo_contrast",   XINE_PARAM_VO(CONTRAST) },
+  [Control_BRIGHTNESS]		= { 32768,       0,  65535,  100,  1000, 0, "gui.vo_brightness",	 "vo_brightness", XINE_PARAM_VO(BRIGHTNESS) },
   [Control_EQ_30]		= {   100,	 0,    100,    1,    10, 0, "gui.eq_30",		 "eq_30",	  XINE_PARAM_EQ_30HZ },
   [Control_EQ_60]		= {   100,	 0,    100,    1,    10, 0, "gui.eq_60",		 "eq_60",	  XINE_PARAM_EQ_60HZ },
   [Control_EQ_125]		= {   100,	 0,    100,    1,    10, 0, "gui.eq_125",		 "eq_125",	  XINE_PARAM_EQ_125HZ },
@@ -266,6 +284,14 @@ GtkObject *ui_register_control_adjustment (control_adjustment_t item)
   return c_adjustments[item];
 }
 
+/* Register a widget for the above status functions */
+
+void ui_register_adjustment_widget (control_adjustment_t item, GtkWidget *widget)
+{
+  g_return_if_fail (item < Control_Adjustments);
+  c_adj_widgets[item] = g_slist_append (c_adj_widgets[item], widget);
+}
+
 /* Set the state of every button in a class, e.g. every play button */
 
 void ui_set_control_button (control_button_t item, gboolean state)
@@ -391,6 +417,31 @@ GtkWidget *ui_button_new_icon_name_mnemonic (const char *name,
   return button;
 }
 
+/* Misc widgets with common properties */
+
+GtkWidget *ui_spin_button_new (GtkObject *adj)
+{
+  return g_object_new (GTK_TYPE_SPIN_BUTTON, "adjustment", adj, "digits", 0,
+		       "update-policy", GTK_UPDATE_ALWAYS, "climb-rate", 1.0,
+		       "numeric", TRUE, NULL);
+}
+
+GtkWidget *ui_hscale_new (GtkObject *adj, GtkPositionType pos, int digits)
+{
+  return g_object_new (GTK_TYPE_HSCALE, "adjustment", adj, "value-pos", pos,
+		       "digits", digits, NULL);
+}
+
+GtkWidget *ui_label_new_with_xalign (const char *label, gfloat x)
+{
+  return g_object_new (GTK_TYPE_LABEL, "label", label, "xalign", x, NULL);
+}
+
+GtkWidget *ui_label_new_with_markup (const char *label)
+{
+  return g_object_new (GTK_TYPE_LABEL, "label", label, "use-markup", TRUE, NULL);
+}
+
 /* Menu/toolbar basics */
 
 GtkUIManager *ui_create_manager (const char *label, GtkWidget *window)
@@ -428,24 +479,6 @@ void gtk_action_group_connect_accelerators (GtkActionGroup *actions)
 
 /* Toolbar control and JS functions */
 
-#define TRIVIAL_GET(what,where) \
-  static void get_##what (se_prop_t *prop, se_prop_read_t *value) \
-  { \
-    value->i = (where); \
-  }
-#define TRIVIAL_GET_SET(what,where,how) \
-  TRIVIAL_GET (what, where) \
-  static void set_##what (int v) \
-  { \
-    if (gtk_toggle_action_get_active (action_items.how) != v) \
-      gtk_action_activate (GTK_ACTION (action_items.how)); \
-  }
-
-TRIVIAL_GET_SET (fs_toolbar, fs_toolbar_visible, fs_toolbar)
-TRIVIAL_GET     (fs_toolbar_top, fs_toolbar_at_top)
-TRIVIAL_GET_SET (wm_toolbar, wm_toolbar_visible, wm_toolbar)
-TRIVIAL_GET_SET (wm_toolbar_snap, wm_toolbar_snap, wm_toolbar_snap)
-
 static void set_fs_toolbar_top (int v)
 {
   if (v)
@@ -477,12 +510,14 @@ static JSBool js_fs_toolbar_show (JSContext *cx, JSObject *obj, uintN argc,
   return JS_TRUE;
 }
 
+#ifdef WITH_OBSOLETE
+
 static JSBool js_fs_toolbar_position (JSContext *cx, JSObject *obj, uintN argc,
 				      jsval *argv, jsval *rval)
 {
   JSBool top;
 
-  se_log_fncall_deprecated ("set_toolbar_position");
+  se_log_fncall_obsolete ("set_toolbar_position");
   se_argc_check_max (1, "set_toolbar_position");
 
   if (argc == 1)
@@ -499,7 +534,8 @@ static JSBool js_fs_toolbar_position (JSContext *cx, JSObject *obj, uintN argc,
   return JS_TRUE;
 }
 
-#endif
+#endif /* WITH_OBSOLETE */
+#endif /* WITH_DEPRECATED */
 
 /* A/V settings JS methods */
 
@@ -532,7 +568,30 @@ static JSBool js_control_reset (JSContext *cx, JSObject *obj, uintN argc,
   return JS_TRUE;
 }
 
-/* Boolean properties */
+/* Integer or boolean properties */
+
+static void ui_prop_get (const se_prop_t *prop, se_prop_read_t *value)
+{
+  const ui_property_t *ui = (void *)prop->data;
+  switch (ui->type)
+  {
+  case UI_PROP_BOOL:
+    value->i = *ui->ui.flag.flag;
+    break;
+  case UI_PROP_XINE_PARAM:
+    value->i = xine_get_param (stream, ui->ui.xine.param);
+    break;
+  case UI_PROP_GTK_WIDGET:
+    value->i = ui->ui.widget.get (*ui->ui.widget.widget);
+    break;
+  case UI_PROP_GTK_VIDEO:
+    value->i = ui->ui.video.get (GTK_VIDEO (gtv));
+    break;
+  case UI_PROP_CUSTOM:
+    ui->ui.custom.get (ui, value);
+    break;
+  }
+}
 
 int ui_property_clip_int (const ui_property_t *prop, int value)
 {
@@ -551,27 +610,47 @@ int ui_property_clip_int (const ui_property_t *prop, int value)
   return value;
 }
 
+static int ui_prop_set_internal (ui_property_t *prop, int data)
+{
+  switch (prop->type)
+  {
+  case UI_PROP_BOOL:
+    if (prop->ui.flag.set)
+      prop->ui.flag.set (data);
+    else if (gtk_toggle_action_get_active (*prop->ui.flag.action) != data)
+      gtk_action_activate (&(*prop->ui.flag.action)->parent);
+    break;
+  case UI_PROP_XINE_PARAM:
+    if (prop->ui.xine.set)
+      prop->ui.xine.set (data);
+    else
+      xine_set_param (stream, prop->ui.xine.param, data);
+    break;
+  case UI_PROP_GTK_WIDGET:
+    prop->ui.widget.set (*prop->ui.widget.widget, data);
+    break;
+  case UI_PROP_GTK_VIDEO:
+    if (gtk_toggle_action_get_active (*prop->ui.video.action) != data)
+      gtk_action_activate (&(*prop->ui.video.action)->parent);
+    break;
+  case UI_PROP_CUSTOM:
+    prop->ui.custom.set (data);
+    break;
+  }
+  return 0;
+}
+
 static int ui_prop_set_int (void *data, se_t *se, se_o_t *o,
 			    se_prop_t *se_prop, se_prop_read_t value)
 {
-  const ui_property_t *prop = data;
-  value.i = ui_property_clip_int (prop, value.i);
-  if (prop->set)
-    prop->set (value.i);
-  else
-    xine_set_param (stream, prop->param, value.i);
-  return 0;
+  value.i = ui_property_clip_int (data, value.i);
+  return ui_prop_set_internal (data, value.i);
 }
 
 static int ui_prop_set_bool (void *data, se_t *se, se_o_t *o,
 			     se_prop_t *se_prop, se_prop_read_t value)
 {
-  const ui_property_t *prop = data;
-  if (prop->set)
-    prop->set (!!value.i);
-  else
-    xine_set_param (stream, prop->param, !!value.i);
-  return 0;
+  return ui_prop_set_internal (data, !!value.i);
 }
 
 static JSBool ui_prop_toggle_bool (JSContext *cx, JSObject *obj,
@@ -603,13 +682,8 @@ void ui_create_properties (const ui_property_t *prop, se_o_t *parent,
     se_o_t *obj =
       se_create_object (gse, parent, prop[i].name, NULL, SE_GROUP_PROPERTIES,
 			gettext (prop[i].help));
-    if (prop[i].param)
-      se_prop_create_xine_param (gse, obj, "v", prop[i].param, type);
-    else
-    {
-      se_prop_create (gse, obj, "v", "", NULL, 0, type, FALSE);
-      se_prop_set_reader_func (gse, obj, "v", prop[i].get);
-    }
+    se_prop_create_with_reader (gse, obj, "v", "", type, (intptr_t) &prop[i],
+				FALSE, ui_prop_get);
 
     if (prop[i].listen.cb)
       se_prop_add_listener (gse, obj, "v", prop[i].listen.cb,
@@ -669,12 +743,21 @@ void ui_preferences_register (xine_t *this)
   const char **vis_labels, *const *vis_src;
   unsigned int i;
 
+  config_version = xine_config_register_num
+    (this, "misc.gxine_config", 0, "gxine internal", NULL, 32767, NULL, NULL);
+  xine_cfg_entry_t entry;
+  if (xine_config_lookup_entry (xine, "misc.gxine_config", &entry))
+  {
+    entry.num_value = CONFIG_VERSION_CURRENT;
+    xine_config_update_entry (xine, &entry);
+  }
+
   /* Register the experience level setting */
   xine_config_register_enum
     (this, "gui.experience_level", 0, experience_labels,
      _("Display of configuration settings"),
      _("Controls whether more advanced configuration settings are shown."),
-     0, NULL, CONFIG_DATA_NONE);
+     0, gtk_true, NULL);
 
   /* Register a few audio/video config items */
   for (i = 0; i < G_N_ELEMENTS (ranges); ++i)
@@ -706,9 +789,10 @@ void ui_preferences_register (xine_t *this)
      40, post_plugins_video_cb, CONFIG_DATA_NONE);
 
   xine_config_register_bool
-    (this, "gui.post_plugins.video_enable", 0,
+    (this, "gui.post_plugins.video_enable", CONFIG_VERSION_ATLEAST (0, 6, 0),
      _("Enable video post-processing at startup"), NULL,
      30, NULL, NULL);
+  config_update_default ("gui.post_plugins.video_enable", 1);
 
   xine_config_register_string
     (this, "gui.post_plugins.audio", "",
@@ -717,37 +801,53 @@ void ui_preferences_register (xine_t *this)
      40, post_plugins_audio_cb, CONFIG_DATA_NONE);
 
   xine_config_register_bool
-    (this, "gui.post_plugins.audio_enable", 0,
+    (this, "gui.post_plugins.audio_enable", CONFIG_VERSION_ATLEAST (0, 6, 0),
      _("Enable audio post-processing at startup"), NULL,
      30, NULL, NULL);
+  config_update_default ("gui.post_plugins.audio_enable", 1);
 
   vis_src = xine_list_post_plugins_typed
 	      (xine, XINE_POST_TYPE_AUDIO_VISUALIZATION);
-  for (i = 0; vis_src[i]; ++i) /**/;
+  int goom = 0;
+  for (i = 0; vis_src[i]; ++i)
+    if (!strcmp (vis_src[i], "goom"))
+      goom = i + 1;
   vis_labels = calloc (i + 2, sizeof (char *));
   vis_labels[0] = N_("None"); /* translate for display, not for config file */
   memcpy (vis_labels + 1, vis_src, (i + 1) * sizeof (char *));
 
   xine_config_register_enum
-    (this, "gui.post_plugins.audio_visualisation", 1, vis_labels,
+    (this, "gui.post_plugins.audio_visualisation", goom, vis_labels,
      _("Default audio visualisation plugin"),
      _("Post-plugin to be used when playing streams without video"),
-     30, NULL, NULL);
+     10, NULL, NULL);
 
   xine_config_register_bool
     (this, "gui.post_plugins.always_show_logo", 1,
      _("In windowed mode, whether the logo is shown after playing an audio-only stream if no audio visualisation was active."), NULL,
-     0, NULL, NULL);
+     0, gtk_true, NULL);
+
+  fs_always_sticky = xine_config_register_bool
+    (this, "gui.fullscreen_mode.always_sticky", 1,
+     _("In full-screen mode, whether the video always appears on all desktops & viewports"),
+     NULL, 0, NULL, NULL);
 
   xine_config_register_enum
-    (this, "gui.fullscreen_toolbar", 1, tbar_pos_labels,
+    (this, "gui.fullscreen_mode.toolbar", 1, tbar_pos_labels,
      _("Default position & visibility of the full-screen toolbar"),
      NULL, 0, NULL, NULL);
 
   xine_config_register_bool
-    (this, "gui.windowedmode_separate_toolbar", 0,
+    (this, "gui.windowed_mode.separate_toolbar", CONFIG_VERSION_ATLEAST (0, 6, 0),
      _("In windowed mode, whether the toolbar is in a separate window"), NULL,
      0, NULL, NULL);
+  config_update_default ("gui.windowed_mode.separate_toolbar", 1);
+
+  xine_config_register_bool
+    (this, "gui.windowed_mode.unblank", CONFIG_VERSION_ATLEAST (0, 6, 0),
+     _("In windowed mode, prevent blanking when playing video"), NULL,
+     0, NULL, NULL);
+  config_update_default ("gui.windowed_mode.unblank", 1);
 
   xine_config_register_bool
     (this, "gui.show_splash", 1,
@@ -810,15 +910,41 @@ static void ui_load_icons (void)
   ui_std_icon (ifactory, GXINE_MEDIA_SPEAKER_MUTE, "nospeaker.png");
   ui_std_icon (ifactory, GXINE_MEDIA_MARK, "/gxine.png");
   ui_std_icon (ifactory, GXINE_LOGO, "/gxine.png");
-//  ui_std_icon (ifactory, "gxine-settings-av-sync", "set-sync.svg");
-//  ui_std_icon (ifactory, "gxine-settings-hue", "set-hue.svg");
-  ui_std_icon (ifactory, "gxine-settings-saturation", "set-saturation.svg");
-  ui_std_icon (ifactory, "gxine-settings-contrast", "set-contrast.svg");
-  ui_std_icon (ifactory, "gxine-settings-brightness", "set-brightness.svg");
-  ui_std_icon (ifactory, "gxine-settings-volume", "set-volume.svg");
-//  ui_std_icon (ifactory, "gxine-settings-compressor", "set-compressor.svg");
-//  ui_std_icon (ifactory, "gxine-settings-amplifier", "set-amplifier.svg");
-#ifdef WITH_DEPRECATED
+
+  static const char *const icons[][2] = {
+    { "gxine-settings-av-sync", "set-sync" },
+    { "gxine-settings-spu", "subtitle" },
+    { "gxine-settings-hue", "set-hue" },
+    { "gxine-settings-saturation", "set-saturation" },
+    { "gxine-settings-contrast", "set-contrast" },
+    { "gxine-settings-brightness", "set-brightness" },
+    { "gxine-settings-volume", "set-volume" },
+//  { "gxine-settings-compressor", "set-compressor" },
+//  { "gxine-settings-amplifier", "set-amplifier" },
+    { "gxine-settings-audio-channel", "set-channel" },
+  };
+
+  /* Do we have an SVG loader? If so, use SVG, else use PNG */
+  GSList *fmt, *fmts = gdk_pixbuf_get_formats ();
+  char *name = NULL;
+  foreach_glist (fmt, fmts)
+  {
+    free (name);
+    name = gdk_pixbuf_format_get_name (fmt->data);
+    if (!strcmp (name, "svg"))
+      break;
+  }
+  free (name);
+  g_slist_free (fmts);
+  int i;
+  for (i = 0; i < G_N_ELEMENTS (icons); ++i)
+  {
+	char name[128];
+    snprintf (name, sizeof(name), "%s.%s", icons[i][1], fmt ? "svg" : "png");
+    ui_std_icon (ifactory, icons[i][0], name);
+  }
+
+#ifdef WITH_OBSOLETE
   ui_std_icon (ifactory, "gxine-logo", "/gxine.png");
 #endif
 
@@ -902,16 +1028,16 @@ void ui_init (void)
   se_o_t *toolbars;
 
   static const ui_property_t tb_props[] = {
-    { "fs", N_("v=bool, toggle(): full-screen toolbar visibility"), 0, get_fs_toolbar, set_fs_toolbar },
-    { "wm", N_("v=bool, toggle(): windowed-mode toolbar visibility"), 0, get_wm_toolbar, set_wm_toolbar },
+    { "fs", N_("v=bool, toggle(): full-screen toolbar visibility"), UI_BOOL (fs_toolbar, fs_toolbar_visible, NULL) },
+    { "wm", N_("v=bool, toggle(): windowed-mode toolbar visibility"), UI_BOOL (wm_toolbar, wm_toolbar_visible, NULL) },
     { NULL }
   };
   static const ui_property_t tb_fs_props[] = {
-    { "at_top", N_("v=bool, toggle(): full-screen toolbar position"), 0, get_fs_toolbar_top, set_fs_toolbar_top },
+    { "at_top", N_("v=bool, toggle(): full-screen toolbar position"), UI_BOOL (fs_toolbar_pos, fs_toolbar_at_top, set_fs_toolbar_top) },
     { NULL }
   };
   static const ui_property_t tb_wm_props[] = {
-    { "snap", N_("v=bool, toggle(): windowed-mode toolbar attachment"), 0, get_wm_toolbar_snap, set_wm_toolbar_snap },
+    { "snap", N_("v=bool, toggle(): windowed-mode toolbar attachment"), UI_BOOL (wm_toolbar_snap, wm_toolbar_snap, NULL) },
     { NULL }
   };
 
@@ -919,8 +1045,10 @@ void ui_init (void)
   static const se_f_def_t defs[] = {
     { "toolbar_show", js_fs_toolbar_show, 0, 0,
       SE_GROUP_HIDDEN, N_("[bool]"), NULL },
+#ifdef WITH_OBSOLETE
     { "set_toolbar_position", js_fs_toolbar_position, 0, 0,
       SE_GROUP_HIDDEN, N_("bool"), N_("at top if true") },
+#endif
     { NULL }
   };
 
@@ -951,7 +1079,7 @@ void ui_init (void)
 			SE_TYPE_BOOL);
   ui_load_icons ();
 
-  if (xine_config_lookup_entry (xine, "gui.fullscreen_toolbar", &entry))
+  if (xine_config_lookup_entry (xine, "gui.fullscreen_mode.toolbar", &entry))
   {
     fs_toolbar_at_top = !(entry.num_value & 1);
     fs_toolbar_visible = (entry.num_value >> 1) & 1;
@@ -983,11 +1111,26 @@ void ui_init (void)
 gboolean ui_post_init (void)
 {
   unsigned int i;
+  uint32_t cap = gtk_video_get_capabilities (gtv);
   gdk_threads_enter ();
   for (i = 0; i < G_N_ELEMENTS (ranges); ++i)
+  {
     if (ranges[i].param && i != Control_VOLUME)
       xine_set_param (stream, ranges[i].param, starts[i]);
+#ifdef VO_CAP_HUE
+    switch (ranges[i].type)
+    {
+    case CAP_VIDEO:
+      if (ranges[i].cap && !(cap & ranges[i].cap))
+        ui_set_list (c_adj_widgets, i, "sensitive", 0);
+      break;
+    default:;
+    }      
+#endif
+  }
   gdk_threads_leave ();
+
+  gxine_init_done ();
   return FALSE;
 }
 
